#!/usr/bin/env python3

import sys
import time
from pathlib import Path
from subprocess import CalledProcessError
from typing import List

if not __name__.startswith("tests.src."):
    sys.path.append(str(Path(__file__).parent.absolute()))
    from OSLayer import OSLayer, TailError
    from CookieCutter import CookieCutter
else:
    from .OSLayer import OSLayer, TailError
    from .CookieCutter import CookieCutter


class BjobsError(Exception):
    pass


class UnknownStatusLine(Exception):
    pass


UNKNOWN = "UNKWN"
ZOMBIE = "ZOMBI"


class StatusChecker:
    SUCCESS = "success"
    RUNNING = "running"
    FAILED = "failed"
    STATUS_TABLE = {
        "PEND": RUNNING,
        "RUN": RUNNING,
        "DONE": SUCCESS,
        "PSUSP": RUNNING,
        "USUSP": RUNNING,
        "SSUSP": RUNNING,
        "WAIT": RUNNING,
        "EXIT": FAILED,
        "POST_DONE": SUCCESS,
        "POST_ERR": FAILED,
    }

    def __init__(
        self,
        jobid: int,
        outlog: str,
        wait_between_tries: float = 0.001,
        max_status_checks: int = 1,
        kill_unknown: bool = False,
        kill_zombie: bool = False,
    ):
        self._jobid = jobid
        self._outlog = outlog
        self.wait_between_tries = wait_between_tries
        self.max_status_checks = max_status_checks
        self.kill_unknown = kill_unknown
        self.kill_zombie = kill_zombie

    @property
    def jobid(self) -> int:
        return self._jobid

    @property
    def outlog(self) -> str:
        return self._outlog

    @property
    def bjobs_query_cmd(self) -> str:
        return "bjobs -o 'stat' -noheader {jobid}".format(jobid=self.jobid)

    def _handle_unknown_job(self) -> str:
        if self.kill_unknown:
            print(
                "[lsf profile warning] {unknown} job status detected for {jobid}. "
                "Killing job...".format(unknown=UNKNOWN, jobid=self.jobid),
                file=sys.stderr,
            )
            self._kill_job()
        # we return running regardless so that the zombie job gets cleaned up
        return self.RUNNING

    def _handle_zombie_job(self) -> str:
        if self.kill_zombie:
            print(
                "[lsf profile warning] {zombie} job status detected for {jobid}. "
                "Killing job...".format(zombie=ZOMBIE, jobid=self.jobid),
                file=sys.stderr,
            )
            self._kill_job()
        # zombie jobs are always considered failed as they don't recover
        return self.FAILED

    def _query_status_using_bjobs(self) -> str:
        output_stream, error_stream = OSLayer.run_process(self.bjobs_query_cmd)

        stdout_is_empty = not output_stream.strip()
        if stdout_is_empty:
            raise BjobsError(
                "bjobs error.\nstdout is empty.\nstderr = {stderr}".format(
                    stderr=error_stream
                )
            )

        if output_stream == UNKNOWN:
            return self._handle_unknown_job()

        if output_stream == ZOMBIE:
            return self._handle_zombie_job()

        return self.STATUS_TABLE[output_stream]

    def _get_tail_of_log_file(self) -> List[str]:
        # 30 lines gives us the whole LSF completion summary
        tail = OSLayer.tail(self.outlog, num_lines=30)
        return [line.decode().strip() for line in tail]

    def _kill_job(self):
        kill_cmd = "bkill -r {}".format(self.jobid)
        _ = OSLayer.run_process(kill_cmd)

    def _query_status_using_log(self) -> str:
        try:
            log_tail = self._get_tail_of_log_file()
        except FileNotFoundError:
            print("Log file {} not found".format(self.outlog), file=sys.stderr)
            return self.FAILED
        except TailError as error:
            print("TailError: {}".format(error), file=sys.stderr)
            return self.FAILED

        try:
            resource_summary_usage_line_index = log_tail.index(
                "Resource usage summary:"
            )
        except ValueError:  # resource usage line not in tail
            return self.RUNNING

        status_line = log_tail[resource_summary_usage_line_index - 2]

        if status_line == "Successfully completed.":
            return self.SUCCESS
        elif status_line.startswith("Exited with exit code"):
            return self.FAILED
        else:
            raise UnknownStatusLine(status_line)

    def get_status(self) -> str:
        status = None
        for _ in range(self.max_status_checks):
            try:
                status = self._query_status_using_bjobs()
                break  # succeeded in getting the status
            except BjobsError as error:
                print(
                    "[Predicted exception] BjobsError: {error}".format(error=error),
                    file=sys.stderr,
                )
                print("Resuming...", file=sys.stderr)
                time.sleep(self.wait_between_tries)
            except KeyError as error:
                print(
                    "[Predicted exception] Unknown job status: {error}".format(
                        error=error
                    ),
                    file=sys.stderr,
                )
                print("Resuming...", file=sys.stderr)
                time.sleep(self.wait_between_tries)
            except CalledProcessError as error:
                print(
                    "[Predicted exception] Error calling bjobs: {error}".format(
                        error=error
                    ),
                    file=sys.stderr,
                )
                print("Resuming...", file=sys.stderr)
                time.sleep(self.wait_between_tries)

        bjobs_failed = status is None
        if bjobs_failed:
            print(
                "bjobs failed {try_times} times. Checking log...".format(
                    try_times=self.max_status_checks
                ),
                file=sys.stderr,
            )
            try:
                status = self._query_status_using_log()
            except UnknownStatusLine as error:
                print("UnknownStatusLine: {}".format(error), file=sys.stderr)
                status = self.FAILED

        return status


if __name__ == "__main__":
    jobid = int(sys.argv[1])
    outlog = sys.argv[2]
    if CookieCutter.get_unknwn_behaviour().lower() == "wait":
        kill_unknown = False
    elif CookieCutter.get_unknwn_behaviour().lower() == "kill":
        kill_unknown = True
    else:
        raise ValueError(
            "Unknown value for {}_behaviour: {}".format(
                UNKNOWN, CookieCutter.get_unknwn_behaviour()
            )
        )
    if CookieCutter.get_zombi_behaviour().lower() == "ignore":
        kill_zombie = False
    elif CookieCutter.get_zombi_behaviour().lower() == "kill":
        kill_zombie = True
    else:
        raise ValueError(
            "Unknown value for {}_behaviour: {}".format(
                ZOMBIE, CookieCutter.get_zombi_behaviour()
            )
        )

    lsf_status_checker = StatusChecker(
        jobid, outlog, kill_unknown=kill_unknown, kill_zombie=kill_zombie
    )
    print(lsf_status_checker.get_status())



#!/usr/bin/env python3
import re
import subprocess as sp
import shlex
import sys
import time
import logging
logger = logging.getLogger("__name__")

STATUS_ATTEMPTS = 20

jobid = sys.argv[1]

for i in range(STATUS_ATTEMPTS):
    try:
        sacct_res = sp.check_output(shlex.split("sacct -P -b -j {} -n".format(jobid)))
        res = {x.split("|")[0]: x.split("|")[1] for x in sacct_res.decode().strip().split("\n")}
        break
    except sp.CalledProcessError as e:
        logger.error("sacct process error")
        logger.error(e)
    except IndexError as e:
        pass
    # Try getting job with scontrol instead in case sacct is misconfigured
    try:
        sctrl_res = sp.check_output(shlex.split("scontrol -o show job {}".format(jobid)))
        m = re.search("JobState=(\w+)", sctrl_res.decode())
        res = {jobid: m.group(1)}
        break
    except sp.CalledProcessError as e:
        logger.error("scontrol process error")
        logger.error(e)
        if i >= STATUS_ATTEMPTS - 1:
            print("failed")
            exit(0)
        else:
            time.sleep(1)

status = res[jobid]

if (status == "BOOT_FAIL"):
    print("failed")
elif (status == "OUT_OF_MEMORY"):
    print("failed")
elif (status.startswith("CANCELLED")):
    print("failed")
elif (status == "COMPLETED"):
    print("success")
elif (status == "DEADLINE"):
    print("failed")
elif (status == "FAILED"):
    print("failed")
elif (status == "NODE_FAIL"):
    print("failed")
elif (status == "PREEMPTED"):
    print("failed")
elif (status == "TIMEOUT"):
    print("failed")
# Unclear whether SUSPENDED should be treated as running or failed
elif (status == "SUSPENDED"):
    print("failed")
else:
    print("running")



#!/usr/bin/env python

import os 
import sys
import subprocess


with os.scandir(sys.argv[1]) as itr:
    i = 0
    for entry in itr:
        bin_id, suffix = os.path.splitext(entry.name)
        if suffix == ".fa":
            i += 1
            subprocess.run('''mv %s %s''' \
                  % (os.path.join(sys.argv[1], entry.name),
                     os.path.join(sys.argv[2] + "." + str(i) + ".fa")), shell=True)


#!/usr/bin/env python3

import glob
import os
import stat
import sys
import subprocess
import concurrent.futures

import pandas as pd
from checkm import prodigal


def run_prodigal(input_list):
    bin_fa = os.path.abspath(input_list[0])
    output_dir = os.path.abspath(input_list[1])

    bin_id = os.path.basename(os.path.splitext(bin_fa)[0])

    pep_file = os.path.join(output_dir, bin_id + ".faa")
    cds_file = os.path.join(output_dir, bin_id + ".ffn")
    gff_file = os.path.join(output_dir, bin_id + ".gff")

    prodigal_runner = prodigal.ProdigalRunner(output_dir)
    prodigal_runner.aaGeneFile = pep_file
    prodigal_runner.ntGeneFile = cds_file
    prodigal_runner.gffFile = gff_file

    best_translation_table = prodigal_runner.run(bin_fa, True)

    if (best_translation_table in [4, 11]) and (os.path.exists(pep_file)) and (os.stat(pep_file)[stat.ST_SIZE]) > 0:
        return (bin_id, bin_fa, pep_file, best_translation_table)
    else:
        return (bin_id, bin_fa, pep_file, f"unknown: {best_translation_table}")


workers = int(sys.argv[1])
input_bins_dir = os.path.dirname(sys.argv[2])
output_done = sys.argv[3]
output_dir = os.path.dirname(output_done)

bin_list = glob.glob(input_bins_dir + "/*.fa")

input_list = []
for bin_fa in bin_list:
    input_list.append((bin_fa, output_dir))

table_list = []


subprocess.run(f'''rm -rf {output_dir}''', shell=True)
subprocess.run(f'''mkdir -p {output_dir}''', shell=True)

with concurrent.futures.ProcessPoolExecutor(max_workers=workers) as executor:
    for table_df in executor.map(run_prodigal, input_list):
        table_list.append(table_df)

table_df = pd.DataFrame(table_list, columns=["bin_id", "bin_file", "pep_file", "best_translation_table"])
table_df.to_csv(output_done, sep="\t", index=False)


#!/usr/bin/env python
import argparse
import os
import re
import pandas as pd
from decimal import *


"""
How to assess the quality of metagenomcis assembly
https://www.biostars.org/p/128629/#128639

Brian Bushnell saied:
calculate the percentage of reads that map back to the assembly
if only 50% of your reads map to the assembly, it is not very complete
but if 95% of your reads map to the assembly, then even if it is
somewhat fragmented, that's probably very good

It might also help to look at the percentage properly paired reads to
detect any chimeras, something that seems especially relevant in a
metagenome assembly

the most useful tools is quast, quality assessment tool for
genome ascalculate map rate from bamsemblies

http://genomebio.org/alignment-stats-bwa/
getting alignment stats out of bwa

bwa mem -t 6 ref read.1.fq read.2.fq \
| samtools view -@6 -Sbh - \
| tee >(samtools flagstat - > stats.out) > aln.bam

http://www.pnas.org/content/pnas/113/42/11901.full.pdf
Deep sequencing of 10,000 human genomes(Amalio Telenti and J.Craig Venter)

metabat_coverage
concoct_coverage
checkm_coverage
"""


def flagstats_summary(flagstats, method, **kwargs):
    """
    get alignment rate from sorted bam file
    samtools flagstat --threads 8 sample.sort.bam
    """
    mapping_info = []
    getcontext().prec = 8

    # with open(flagstat_list, 'r') as list_handle:
    if method == 1:
        list_handle = open(flagstats, "r")
    if method == 2:
        list_handle = flagstats

    for flagstat_file in list_handle:
        if os.path.exists(flagstat_file.strip()):
            info = {}
            info["sample_id"] = os.path.basename(flagstat_file.strip()).split(".")[0]
            stat_list = open(flagstat_file.strip(), "r").readlines()

            info["total_num"] = stat_list[0].split(" ")[0]

            if len(stat_list) == 13:
                info["read_1_num"] = stat_list[6].split(" ")[0]
                info["read_2_num"] = stat_list[7].split(" ")[0]

                mapped = re.split(r"\(|\s+", stat_list[4])
                info["mapped_num"] = mapped[0]
                info["mapped_rate"] = Decimal(mapped[5].rstrip("%")) / Decimal(100)
            
                #primary_mapped = re.split(r"\(|\s+", stat_list[5])
                #info["primary_mapped_num"] = primary_mapped[0]
                #info["primary_mapped_rate"] = Decimal(primary_mapped[6].rstrip("%")) / Decimal(100)

            elif len(stat_list) == 16:
                info["read_1_num"] = stat_list[9].split(" ")[0]
                info["read_2_num"] = stat_list[10].split(" ")[0]

                mapped = re.split(r"\(|\s+", stat_list[6])
                info["mapped_num"] = mapped[0]
                info["mapped_rate"] = Decimal(mapped[5].rstrip("%")) / Decimal(100)
            
                primary_mapped = re.split(r"\(|\s+", stat_list[7])
                info["primary_mapped_num"] = primary_mapped[0]
                info["primary_mapped_rate"] = Decimal(primary_mapped[6].rstrip("%")) / Decimal(100)
 
            paired = re.split(r"\(|\s+", stat_list[-5])
            info["paired_num"] = paired[0]
            paired_rate = paired[6].rstrip("%")
            if paired_rate != "N/A":
                info["paired_rate"] = Decimal(paired_rate) / Decimal(100)
                info["mapping_type"] = "paired-end"
            else:
                info["paired_rate"] = paired_rate
                info["mapping_type"] = "single-end"

            singletons = re.split(r"\(|\s+", stat_list[-3])
            info["singletons_num"] = singletons[0]
            singletons_rate = singletons[5].rstrip("%")
            if singletons_rate != "N/A":
                info["singletons_rate"] = Decimal(singletons_rate) / Decimal(100)
            else:
                info["singletons_rate"] = singletons_rate

            info["mate_mapped_num"] = re.split(r"\(|\s+", stat_list[-2])[0]
            info["mate_mapped_num_mapQge5"] = re.split(r"\(|\s+", stat_list[-1])[0]
            mapping_info.append(info)

    mapping_info_df = pd.DataFrame(mapping_info)
    if "output" in kwargs:
        mapping_info_df.to_csv(kwargs["output"], sep="\t", index=False)
    return mapping_info_df


def main():
    """main funciton"""
    parser = argparse.ArgumentParser(
        description="compute alignment rate based bam file"
    )
    parser.add_argument(
        "-statlist", default=None, type=str, help="sorted bam file list"
    )
    parser.add_argument("-statfiles", default=None, nargs="*", help="sorted bam file")
    parser.add_argument("-outfile", type=str, help="output alignment rate file")
    args = parser.parse_args()
    if args.statlist:
        method = 1
        flagstats_summary(args.statlist, method, output=args.outfile)
    if args.statfiles:
        method = 2
        flagstats_summary(args.statfiles, method, output=args.outfile)


if __name__ == "__main__":
    main()



#!/usr/bin/env python3

import time
import gzip
import os
import subprocess

import pandas as pd
from Bio import bgzf


def gtdbtk_prepare(rep_table, batch_num, bins_dir):
    os.makedirs(bins_dir, exist_ok=True)

    table_df = pd.read_csv(rep_table, sep="\t")

    batchid = -1
    if len(table_df) > 0:
        for batch in range(0, len(table_df), batch_num):
            batchid += 1
            columns_list = list(table_df.columns)
            bin_file_index = columns_list.index("bin_file")
            genome_index = columns_list.index("genome")
            best_translation_table_index = columns_list.index("best_translation_table")
            table_split = table_df.iloc[batch:batch+batch_num,
                                        [bin_file_index, genome_index, best_translation_table_index]]
            table_split.to_csv(os.path.join(bins_dir, f"bins_input.{batchid}.tsv"),
                               sep="\t", index=False, header=None)
    else:
        subprocess.run(f'''touch {os.path.join(bins_dir, "bins_input.0.tsv")}''', shell=True)


def demultiplex(kraken2_output, r1, r2, change_seq_id, prefix, log=None):
    start_time = time.time()
    taxid_counter = {}
    demultiplexer = {}

    with open(kraken2_output, "r") as kh:
        for line in kh:
            cols = line.split("\t")
            read_id = cols[1]
            tax_name = cols[2].split("(")[0].strip()
            tax_id = int(cols[2].split("(")[-1].split(")")[0].split()[-1])

            demultiplexer[read_id] = tax_id
            if tax_id in taxid_counter:
                taxid_counter[tax_id][1] += 1
            else:
                taxid_counter[tax_id] = [tax_name, 1]
    if log is not None:
        log_h.write(
            "step_1: parse kraken2 output has spent %d s\n" % (time.time() - start_time)
        )

    start_time = time.time()
    gzip_h = {}
    for i in taxid_counter:
        gzip_h[i] = {}
        gzip_h[i]["r1"] = bgzf.BgzfWriter(prefix + ".%d.1.fq.gz" % i, "wb")
        gzip_h[i]["r2"] = bgzf.BgzfWriter(prefix + ".%d.2.fq.gz" % i, "wb")

    if r1.endswith(".gz"):
        r1_h = gzip.open(r1, "rt")
        r2_h = gzip.open(r2, "rt")
    else:
        r1_h = open(r1, "rt")
        r2_h = open(r2, "rt")

    if change_seq_id:
        sample_tag = os.path.basename(prefix)

    if log is not None:
        log_h.write("step_2: begin demultiplex taxid-reads\n")
    for read_1, read_2 in zip(r1_h, r2_h):
        read_id = read_1[1:].split("/")[0]
        if change_seq_id:
            gzip_h[demultiplexer[read_id]]["r1"].write(
                ">%s|%s%s%s%s"
                % (sample_tag, read_1[1:], next(r1_h), next(r1_h), next(r1_h))
            )
            gzip_h[demultiplexer[read_id]]["r2"].write(
                ">%s|%s%s%s%s"
                % (sample_tag, read_2[1:], next(r2_h), next(r2_h), next(r2_h))
            )
        else:
            gzip_h[demultiplexer[read_id]]["r1"].write(
                "%s%s%s%s" % (read_1, next(r1_h), next(r1_h), next(r1_h))
            )
            gzip_h[demultiplexer[read_id]]["r2"].write(
                "%s|%s%s%s%s" % (read_2, next(r2_h), next(r2_h), next(r2_h))
            )
    if log is not None:
        log_h.write(
            "step_2: demultiplex taxid-reads has spent %d s\n"
            % (time.time() - start_time)
        )



#!/usr/bin/env python3

import os
import subprocess
import resource
from Bio import SeqIO
import pandas as pd


def get_binning_info(bins_dir, cluster_file, assembler):
    if assembler.lower() in ["spades", "metaspades", "megahit"]:
        with os.scandir(bins_dir) as itr, open(cluster_file, "w") as oh:
            for entry in itr:
                bin_id, suffix = os.path.splitext(entry.name)
                if suffix == ".fa":
                    cluster_num = bin_id.split(".")[-1]
                    bin_fa = os.path.join(bins_dir, entry.name)
                    for seq in SeqIO.parse(bin_fa, "fasta"):
                        # graphbin
                        # oh.write("%s,%s" %
                        #         ("_".join(seq.id.split("_")[:2]), cluster_num))
                        # graphbin 2
                        oh.write(f"{seq.id},{cluster_num}\n")


def generate_bins(cluster_file, scaftigs, prefix):

    def get_accession(identifier):
        return "_".join(identifier.split("_")[:2])

    # graphbin
    # scaftigs_index = SeqIO.index(scaftigs, "fasta", key_function=get_accession)

    # graphbin2
    scaftigs_index = SeqIO.index(scaftigs, "fasta")

    df = pd.read_csv(cluster_file, names=["scaftigs_id", "bin_id"])\
           .astype({"scaftigs_id": str,
                    "bin_id": str})\
           .set_index("bin_id")

    for i in df.index.unique():
        scaftigs_id_list = df.loc[[i], "scaftigs_id"]\
                             .dropna().tolist()
        bin_fa = prefix + "." + i + ".fa"
        with open(bin_fa, 'w') as oh:
            for scaftigs_id in scaftigs_id_list:
                SeqIO.write(scaftigs_index[scaftigs_id], oh, "fasta")


def extract_bins_report(bins_report_table):
    if os.path.getsize(bins_report_table) > 0:
        bins_report = pd.read_csv(bins_report_table, sep='\t', header=[0, 1])\
                        .rename(columns={
                            "Unnamed: 0_level_1": "assembly_group",
                            "Unnamed: 1_level_1": "bin_id",
                            "Unnamed: 2_level_1": "bin_file",
                            "Unnamed: 3_level_1": "assembler",
                            "Unnamed: 4_level_1": "binner"}, level=1)

        bins_report = bins_report[[
            ("assembly_group", "assembly_group"),
            ("bin_id", "bin_id"),
            ("bin_file", "bin_file"),
            ("assembler", "assembler"),
            ("binner", "binner"),
            ("length", "sum"),
            ("length", "N50")]]
    
        bins_report.columns = ["assembly_group", "bin_id", "bin_file", "assembler", "binner", "length", "N50"]
        return bins_report
    else:
        return pd.DataFrame(columns=["assembly_group", "bin_id", "bin_file", "assembler", "binner", "length", "N50"])


'''
            table_bins = pd.read_csv(input.table_bins, sep="\t", header=[0, 1])
            table_bins = table_bins[
                [
                    ("bin_id", "Unnamed: 1_level_1"),
                    ("chr", "count"),
                    ("length", "sum"),
                    ("length", "min"),
                    ("length", "max"),
                    ("length", "std"),
                    ("length", "N50")
                ]
            ]
            table_bins.columns = [
                "user_genome",
                "contig_number",
                "contig_length_sum",
                "contig_length_min",
                "contig_length_max",
                "contig_length_std",
                "N50"
            ]
'''


def combine_jgi(jgi_list, output_file):
    #first = False
    #jgi_df_list = []
    #for jgi in input.jgi:
    #    if not first:
    #        # jgi format
    #        # contigName\tcontigLen\ttotalAvgDepth\t{sample_id}.align2combined_scaftigs.sorted.bam
    #        jgi_df_first = pd.read_csv(jgi, sep="\t")\
    #                     .loc[:, ["contigName", "contigLen", "totalAvgDepth"]]\
    #                     .dtype({"contigName": str, "contigLen": np.int32, "totalAvgDepth": np.float32})\
    #                     .set_index("contigName")
    #        jgi_df = pd.read_csv(jgi, sep="\t").iloc[:, [0, 3]]\
    #                   .dtype({"contigName": str})
    #        jgi_df[jgi_df.columns[1]] = jgi_df[jgi_df.columns[1]].astype(np.float32)
    #        jgi_df_list = [jgi_df_first, jgi_df.set_index("contigName")]
    #        first = True
    #    else:
    #        jgi_df = pd.read_csv(jgi, sep="\t").iloc[:, [0, 3]]\
    #                   .dtype({"contigName": str})
    #        jgi_df[jgi_df.columns[1]] = jgi_df[jgi_df.columns[1]].astype(np.float32)
    #        jgi_df_list.append(jgi_df.set_index("contigName"))
    ## big table, huge memory
    #pd.concat(jgi_df_list, axis=1).reset_index().to_csv(output.matrix, sep="\t", index=False)

    #matrix_list = []
    #for jgi in input.jgi:
    #    if not first:
    #        first = True
    #        with open(jgi, 'r') as ih:
    #            for line in ih:
    #                line_list = line.strip().split("\t")
    #                matrix_list.append(line_list)
    #    else:
    #        with open(jgi, 'r') as ih:
    #            count = -1
    #            for line in ih:
    #                count += 1
    #                line_list = line.strip().split("\t")
    #                matrix_list[count].append(line_list[3])

    #with open(output.matrix, 'w') as oh:
    #    for i in matrix_list:
    #        oh.write("\t".join(i) + "\n")

    # aovid OSError: Too many open files
    max_num_file = resource.getrlimit(resource.RLIMIT_NOFILE)[0]
    if len(jgi_list) > max_num_file:
        max_num_file += len(jgi_list)
        resource.setrlimit(resource.RLIMIT_NOFILE, (max_num_file, max_num_file))

    outdir = os.path.dirname(output_file)
    os.makedirs(outdir, exist_ok=True)

    files_handle = []
    for jgi in jgi_list:
        files_handle.append(open(jgi, 'r'))

    with open(output_file, 'w') as oh:
        for line in files_handle[0]:
            oh.write(line.strip())
            for handle in files_handle[1:]:
                depth = handle.readline().strip().split("\t")[3]
                oh.write(f'''\t{depth}''')
            oh.write("\n")

    for handle in files_handle:
        handle.close()


class CookieCutter:
    """
    Cookie Cutter wrapper
    """

    @staticmethod
    def get_default_threads() -> int:
        return int("1")

    @staticmethod
    def get_default_mem_mb() -> int:
        return int("1024")

    @staticmethod
    def get_log_dir() -> str:
        return "logs/cluster"

    @staticmethod
    def get_default_queue() -> str:
        return ""

    @staticmethod
    def get_lsf_unit_for_limits() -> str:
        return "KB"

    @staticmethod
    def get_unknwn_behaviour() -> str:
        return "wait"

    @staticmethod
    def get_zombi_behaviour() -> str:
        return "ignore"

    @staticmethod
    def get_latency_wait() -> float:
        return float("5")



#!/usr/bin/env python3

import sys
from metapi import tooler
import seaborn as sns
import matplotlib.pyplot as plt
import argparse
import os
import json
import pandas as pd
import numpy as np

import matplotlib

matplotlib.use("agg")


def parse_fastp_json(json_f, paired):
    trimming_dict = {}
    sample_id = os.path.basename(json_f).split(".")[0]
    with open(json_f, "r") as ih:
        json_ob = json.load(ih)
        trimming_dict["sample_id"] = sample_id
        trimming_dict["before_filtering_total_reads"] = json_ob["summary"][
            "before_filtering"
        ]["total_reads"]
        trimming_dict["before_filtering_total_bases"] = json_ob["summary"][
            "before_filtering"
        ]["total_bases"]
        trimming_dict["before_filtering_q20_bases"] = json_ob["summary"][
            "before_filtering"
        ]["q20_bases"]
        trimming_dict["before_filtering_q30_bases"] = json_ob["summary"][
            "before_filtering"
        ]["q30_bases"]
        trimming_dict["before_filtering_q20_rate"] = json_ob["summary"][
            "before_filtering"
        ]["q20_rate"]
        trimming_dict["before_filtering_q30_rate"] = json_ob["summary"][
            "before_filtering"
        ]["q30_rate"]
        trimming_dict["before_filtering_qc_content"] = json_ob["summary"][
            "before_filtering"
        ]["qc_content"]
        trimming_dict["before_filtering_read1_mean_length"] = json_ob["summary"][
            "before_filtering"
        ]["read1_mean_length"]
        if paired:
            trimming_dict["before_filtering_read2_mean_length"] = json_ob["summary"][
                "before_filtering"
            ]["read2_mean_length"]

        trimming_dict["read1_before_filtering_total_reads"] = json_ob[
            "read1_before_filtering"
        ]["total_reads"]
        trimming_dict["read1_before_filtering_total_bases"] = json_ob[
            "read1_before_filtering"
        ]["total_bases"]
        trimming_dict["read1_before_filtering_q20_bases"] = json_ob[
            "read1_before_filtering"
        ]["q20_bases"]
        trimming_dict["read1_before_filtering_q30_bases"] = json_ob[
            "read1_before_filtering"
        ]["q30_bases"]
        if paired:
            trimming_dict["read2_before_filtering_total_reads"] = json_ob[
                "read2_before_filtering"
            ]["total_reads"]
            trimming_dict["read2_before_filtering_total_bases"] = json_ob[
                "read2_before_filtering"
            ]["total_bases"]
            trimming_dict["read2_before_filtering_q20_bases"] = json_ob[
                "read2_before_filtering"
            ]["q20_bases"]
            trimming_dict["read2_before_filtering_q30_bases"] = json_ob[
                "read2_before_filtering"
            ]["q30_bases"]

        trimming_dict["after_filtering_total_reads"] = json_ob["summary"][
            "after_filtering"
        ]["total_reads"]
        trimming_dict["after_filtering_total_bases"] = json_ob["summary"][
            "after_filtering"
        ]["total_bases"]
        trimming_dict["after_filtering_q20_bases"] = json_ob["summary"][
            "after_filtering"
        ]["q20_bases"]
        trimming_dict["after_filtering_q30_bases"] = json_ob["summary"][
            "after_filtering"
        ]["q30_bases"]
        trimming_dict["after_filtering_q20_rate"] = json_ob["summary"][
            "after_filtering"
        ]["q20_rate"]
        trimming_dict["after_filtering_q30_rate"] = json_ob["summary"][
            "after_filtering"
        ]["q30_rate"]
        trimming_dict["after_filtering_qc_content"] = json_ob["summary"][
            "after_filtering"
        ]["qc_content"]
        trimming_dict["after_filtering_read1_mean_length"] = json_ob["summary"][
            "after_filtering"
        ]["read1_mean_length"]
        if paired:
            trimming_dict["after_filtering_read2_mean_length"] = json_ob["summary"][
                "after_filtering"
            ]["read2_mean_length"]

        trimming_dict["read1_after_filtering_total_reads"] = json_ob[
            "read1_after_filtering"
        ]["total_reads"]
        trimming_dict["read1_after_filtering_total_bases"] = json_ob[
            "read1_after_filtering"
        ]["total_bases"]
        trimming_dict["read1_after_filtering_q20_bases"] = json_ob[
            "read1_after_filtering"
        ]["q20_bases"]
        trimming_dict["read1_after_filtering_q30_bases"] = json_ob[
            "read1_after_filtering"
        ]["q30_bases"]
        if paired:
            trimming_dict["read2_after_filtering_total_reads"] = json_ob[
                "read2_after_filtering"
            ]["total_reads"]
            trimming_dict["read2_after_filtering_total_bases"] = json_ob[
                "read2_after_filtering"
            ]["total_bases"]
            trimming_dict["read2_after_filtering_q20_bases"] = json_ob[
                "read2_after_filtering"
            ]["q20_bases"]
            trimming_dict["read2_after_filtering_q30_bases"] = json_ob[
                "read2_after_filtering"
            ]["q30_bases"]

        return trimming_dict


def change(input_file, output_file, sample_id, step, fq_type, reads_list):
    df = pd.read_csv(input_file, sep="\t").sort_values("file", ascending=True)
    df["id"] = sample_id
    df["reads"] = reads_list
    df["step"] = step
    df["fq_type"] = fq_type
    df.to_csv(output_file, sep="\t", index=False)


def compute_host_rate(df, steps, samples_id_list, allow_miss_samples=True, **kwargs):
    all_state_set = set()
    have_state_set = set()
    sample_reads = {}
    host_rate = {}

    for step in steps:
        for sample_id in samples_id_list:
            all_state_set.add((sample_id, step))

    df = df.set_index("id")
    for sample_id in df.index.unique():
        if not pd.isnull(sample_id):
            sample_reads[sample_id] = {}

            for step in steps:
                step_df = df.loc[
                    [sample_id],
                ].query(f'''reads=="fq1" and step=="{step}"''')
                if not step_df.empty:
                    num_seqs = step_df["num_seqs"].to_list()[0]
                    #print(num_seqs)
                    if num_seqs >= 0:
                        #print(sample_id)
                        have_state_set.add((sample_id, step))
                        sample_reads[sample_id][step] = num_seqs
                    else:
                        print(f'''WARNING: {sample_id}_{step}_num_seqs: {num_seqs}''')
        else:
            print("WARNING: found NA sample id in qcreport summary")

    not_have_state_set = all_state_set - have_state_set

    #print("\n")
    #print(all_state_set)
    #print("\n")
    #print(have_state_set)
    #print("\n")
    #print(not_have_state_set)
    #print("\n")

    if len(not_have_state_set) > 0:
        for j in not_have_state_set:
            print(f"""WARNING: there are no {j[0]} full stats report for {j[1]} step""")
        if not allow_miss_samples:
            print("Please check stats report again for each sample")
            sys.exit(1)
    
    for sample_id in df.index.unique():
        if not pd.isnull(sample_id):
            if sample_id in sample_reads:
                if "rmhost" in steps:
                    if "trimming" in steps:
                        if ("trimming" in sample_reads[sample_id]) and ("rmhost" in sample_reads[sample_id]):
                            host_rate[sample_id] = (
                                sample_reads[sample_id]["trimming"]
                                - sample_reads[sample_id]["rmhost"]
                                ) / sample_reads[sample_id]["trimming"]
                        else:
                            host_rate[sample_id] = np.nan
                    elif "raw" in steps:
                        if ("raw" in sample_reads[sample_id]) and ("rmhost" in sample_reads[sample_id]):
                            host_rate[sample_id] = (
                                sample_reads[sample_id]["raw"]
                                - sample_reads[sample_id]["rmhost"]
                                ) / sample_reads[sample_id]["raw"]
                        else:
                            host_rate[sample_id] = np.nan
                else:
                    host_rate[sample_id] = np.nan
            else:
                host_rate[sample_id] = np.nan

    df = df.reset_index()
    df["host_rate"] = df.apply(lambda x: host_rate[x["id"]], axis=1)

    if "output" in kwargs:
        df.to_csv(kwargs["output"], sep="\t", index=False)
    return df


def qc_bar_plot(df, engine, stacked=False, **kwargs):
    if engine == "seaborn":
        # seaborn don't like stacked barplot
        f, ax = plt.subplots(figsize=(10, 7))
        df_ = df.query('reads=="fq1"')
        sns.barplot(x="id", y="num_seqs", hue="step", data=df_)
        ax.set_xticklabels(ax.get_xticklabels(), rotation=-90)

    elif engine == "pandas":
        if not stacked:
            df_ = (
                df.query('reads=="fq1"')
                .pivot(index="id", columns="step", values="num_seqs")
                .loc[:, ["raw", "trimming", "rmhost"]]
            )
            df_.plot(kind="bar", figsize=(10, 7))

        else:
            dict_ = {"id": [], "clean": [], "rmhost": [], "trim": []}
            df = df.set_index("id")
            for i in df.index.unique():
                reads_total = 0

                reads_trimmed = 0
                reads_host = 0
                reads_clean = 0

                if (
                    not df.loc[
                        [i],
                    ]
                    .query('reads=="fq1" and step=="raw"')
                    .empty
                ):
                    reads_total = df.loc[[i],].query('reads=="fq1" and step=="raw"')[
                        "num_seqs"
                    ][0]

                if (
                    not df.loc[
                        [i],
                    ]
                    .query('reads=="fq1" and step=="trimming"')
                    .empty
                ):
                    reads_trim = df.loc[[i],].query(
                        'reads=="fq1" and step=="trimming"'
                    )["num_seqs"][0]

                if (
                    not df.loc[
                        [i],
                    ]
                    .query('reads=="fq1" and step=="rmhost"')
                    .empty
                ):
                    reads_clean = df.loc[[i],].query('reads=="fq1" and step=="rmhost"')[
                        "num_seqs"
                    ][0]

                reads_trimmed = reads_total - reads_trim
                reads_host = reads_trim - reads_clean

                dict_["id"].append(i)
                dict_["trim"].append(reads_trimmed)
                dict_["rmhost"].append(reads_host)
                dict_["clean"].append(reads_clean)

            df_ = pd.DataFrame(dict_).sort_values("id").set_index("id")

            colors = ["#2ca02c", "#ff7f0e", "#1f77b4"]

            df_.plot(kind="bar", stacked=True, color=colors, figsize=(10, 7))

    plt.xlabel("Sample ID")
    plt.ylabel("The number of reads(-pair)")
    plt.title("Fastq quality control barplot", fontsize=11)

    if "output" in kwargs:
        plt.savefig(kwargs["output"])


def main():
    parser = argparse.ArgumentParser(description="quality control reporter")
    parser.add_argument("--raw_stats_list", help="raw stats list")
    parser.add_argument("--trimming_stats_list", help="trimming stats list")
    parser.add_argument("--rmhost_stats_list", help="rmhost stats list")
    parser.add_argument("--output", help="quality control output basename")
    args = parser.parse_args()

    raw_list = pd.read_csv(args.raw_stats_list, header=None, names=["raw"])
    trimming_list = pd.read_csv(
        args.trimming_stats_list, header=None, names=["trimming"]
    )
    rmhost_list = pd.read_csv(args.rmhost_stats_list, header=None, names=["rmhost"])

    df = tooler.merge(
        raw_list["raw"].dropna().tolist()
        + trimming_list["trimming"].dropna().tolist()
        + rmhost_list["rmhost"].dropna().tolist(),
        tooler.parse,
        8,
    )

    df_ = compute_host_rate(
        df,
        ["raw", "trimming", "rmhost"],
        df.set_index("id").index.unique(),
        allow_miss_samples=True,
        output=os.path.join(args.output, ".stats.tsv"),
    )
    qc_bar_plot(df_, "seaborn", output=os.path.join(args.output, ".plot.pdf"))


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
import os
import sys
import argparse
import subprocess

from snakemake.utils import read_job_properties

parser=argparse.ArgumentParser(add_help=False)
parser.add_argument("--depend", help="Space separated list of ids for jobs this job should depend on.")
parser.add_argument("-a", help="Declare the time when the job becomes eligible for execution.")
parser.add_argument("-A", help="Define the account string.")
parser.add_argument("-b", help="PBS Server timeout.")
parser.add_argument("-c", help="Checkpoint options.")
parser.add_argument("-C", help="Directive prefix in script file.")
parser.add_argument("-d", help="Working directory to be used (default: ~). PBS_O_INITDIR")
parser.add_argument("-D", help="Root directory to be used. PBS_O_ROOTDIR")
parser.add_argument("-e", help="standard error path.")
parser.add_argument("-f", help="Fault tolerant.",action="store_true")
parser.add_argument("-h", help="Apply user hold at submission time",action="store_true")
parser.add_argument("-j", help="Merge standard error and standard out. (oe or eo)")
parser.add_argument("-l", help="Resource list.")
parser.add_argument("-m", help="Mail options.")
parser.add_argument("-M", help="Mail users.")
parser.add_argument("-N", help="Name for the job.")
parser.add_argument("-o", help="standard output path.")
parser.add_argument("-p", help="Set job priority.")
parser.add_argument("-P", help="Proxy user for job.")
parser.add_argument("-q", help="Set destination queue.")
parser.add_argument("-t", help="Array request.")
parser.add_argument("-u", help="Set user name for job.")
parser.add_argument("-v", help="Environment variables to export to the job.")
parser.add_argument("-V", help="Export all environment variables.",action="store_true")
parser.add_argument("-w", help="Set working directory. PBS_O_WORKDIR")
parser.add_argument("-W", help="Additional attributes.")
parser.add_argument("--help", help="Display help message.",action="store_true")

parser.add_argument("positional",action="append",nargs="?")
args = parser.parse_args()

if args.help :
    parser.print_help()
    sys.exit(0)

jobscript = sys.argv[-1]

job_properties = read_job_properties(jobscript)

atime=""
acc_string=""
pbs_time=""
chkpt=""
pref=""
dd=""
rd=""
se=""
ft=""
hold=""
j=""
resource=""
mail=""
mailuser=""
jname=""
so=""
priority=""
proxy=""
q=""
ar=""
user=""
ev=""
eall=""
wd=""
add=""
depend=""
resourceparams=""
extras=""


if args.depend:
	for m in args.depend.split(" "):
		depend = depend + ":" + m
if depend:
	depend = " -W \"depend=afterok" + depend + "\""

if args.positional:
	for m in args.positional:
		extras = extras + " " + m

if args.a: atime = " -a " + args.a
if args.A: acc_string = " -A " + args.A
if args.b: pbs_time = " -b " + args.b
if args.c: chkpt = " -c " + args.c
if args.C: pref = " -C " + args.C
if args.d: dd = " -d " + args.d
if args.D: rd = " -D " + args.D
if args.e: se = " -e " + args.e
if args.f: ft = " -f"
if args.h: hold = " -h"
if args.j: j = " -j " + args.j
if args.l: resource = " -l " + args.l
if args.m: mail = " -m " + args.m
if args.M: mailuser = " -M " + args.M
if args.N: jname = " -N " + args.N
if args.o: so = " -o " + args.o
if args.p: priority = " -p " + args.p
if args.P: proxy = " -P " + args.P
if args.q: q = " -q " + args.q
if args.t: ar = " -t " + args.ar
if args.u: user = " -u " + args.u
if args.v: ev = " -v " + args.v
if args.V: eall = " -V"
if args.w: wd = " -w " + args.w
if args.W: add= " -W \"" + args.W + "\""

nodes=""
ppn=""
mem=""
walltime=""

if "threads" in job_properties:
    ppn = "ppn=" + str(job_properties["threads"])

if "resources" in job_properties:
    resources = job_properties["resources"]
    if "nodes" in resources: nodes="nodes=" + str(resources["nodes"])
    if ppn and not nodes : nodes="nodes=1"
    if "mem" in resources: mem="mem=" + str(resources["mem"])
    if "walltime" in resources: walltime="walltime=" + str(resources["walltime"])

if nodes or ppn or mem or walltime: resourceparams = " -l \""
if nodes: resourceparams = resourceparams + nodes
if nodes and ppn: resourceparams = resourceparams + ":" + ppn
if nodes and mem: resourceparams = resourceparams + ","
if mem: resourceparams = resourceparams + mem
if walltime and (nodes or mem): resourceparams = resourceparams + ","
if walltime: resourceparams = resourceparams + walltime
if nodes or mem or walltime: resourceparams = resourceparams + "\""

if "cluster" in job_properties:
    cluster = job_properties["cluster"]
    if "error" in cluster:
        os.makedirs(os.path.dirname(cluster["error"]), exist_ok=True)
        se = " -e " + cluster["error"]
    if "output" in cluster:
        os.makedirs(os.path.dirname(cluster["output"]), exist_ok=True)
        so = " -o " + cluster["output"]

cmd = "qsub {a}{A}{b}{c}{C}{d}{D}{e}{f}{h}{j}{l}{m}{M}{N}{o}{p}{P}{q}{t}{u}{v}{V}{w}{W}{rp}{dep}{ex}".format(\
	a=atime,A=acc_string,b=pbs_time,c=chkpt,C=pref,d=dd,D=rd,e=se,f=ft,h=hold,j=j,l=resource,m=mail,M=mailuser,\
	N=jname,o=so,p=priority,P=proxy,q=q,t=ar,u=user,v=ev,V=eall,w=wd,W=add,rp=resourceparams,dep=depend,ex=extras)

try:
    res = subprocess.run(cmd, check=True, shell=True, stdout=subprocess.PIPE)
except subprocess.CalledProcessError as e:
    raise e

res = res.stdout.decode()
print(res.strip())



#!/usr/bin/env python3
import re
import subprocess as sp
import shlex
import sys
import time
import logging

logger = logging.getLogger("__name__")
logger.setLevel(40)

STATUS_ATTEMPTS = 20

jobid = int(sys.argv[1])
job_status = "running"

# WARNING this currently has no support for task array jobs

for i in range(STATUS_ATTEMPTS):
    # first try qstat to see if job is running
    # we can use `qstat -s pr -u "*"` to check for all running and pending jobs
    try:
        qstat_res = sp.check_output(shlex.split(f"qstat -s pr")).decode().strip()

        # skip the header using [2:]
        res = {
            int(x.split()[0]) : x.split()[4] for x in qstat_res.splitlines()[2:]
        }

        # job is in an unspecified error state
        if "E" in res[jobid]:
            job_status = "failed"
            break

        job_status = "running"
        break

    except sp.CalledProcessError as e:
        logger.error("qstat process error")
        logger.error(e)
    except KeyError as e:
        # if the job has finished it won't appear in qstat and we should check qacct
        # this will also provide the exit status (0 on success, 128 + exit_status on fail)
        # Try getting job with scontrol instead in case sacct is misconfigured
        try:
            qacct_res = sp.check_output(shlex.split(f"qacct -j {jobid}"))

            exit_code = int(re.search("exit_status  ([0-9]+)", qacct_res.decode()).group(1))

            if exit_code == 0:
                job_status = "success"
                break

            if exit_code != 0:
                job_status = "failed"
                break

        except sp.CalledProcessError as e:
            logger.warning("qacct process error")
            logger.warning(e)
            if i >= STATUS_ATTEMPTS - 1:
                job_status = "failed"
                break
            else:
                # qacct can be quite slow to update on large servers
                time.sleep(5)
        pass

print(job_status)



#!/usr/bin/env python3

import sys
import subprocess
import xml.etree.cElementTree as ET

jobid = sys.argv[1]

try:
    res = subprocess.run("qstat -f -x {}".format(jobid), check=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, shell=True)

    xmldoc = ET.ElementTree(ET.fromstring(res.stdout.decode())).getroot()
    job_state = xmldoc.findall('.//job_state')[0].text

    if job_state == "C":
        exit_status = xmldoc.findall('.//exit_status')[0].text
        if exit_status == '0':
            print("success")
        else:
            print("failed")
    else:
        print("running")

except (subprocess.CalledProcessError, IndexError, KeyboardInterrupt) as e:
    print("failed")



#!/usr/bin/env python3

import os
import argparse
import subprocess
import sys


def link_or_cat(args):
    fq_gz = os.path.join(args.output_dir, args.basename + ".fq.gz")

    if (not os.path.exists(fq_gz)) or (os.path.getsize(fq_gz) == 0):
        subprocess.call(
            f'''rm -rf {fq_gz}''',
            shell=True, stdout=sys.stdout, stderr=sys.stderr)

        if len(args.input_file) == 1:
            reads = os.path.realpath(args.input_file[0])
            subprocess.call(
                f'''
                pushd {args.output_dir} && \
                ln -s {reads} {args.basename}.fq.gz && \
                popd
                ''', shell=True, stdout=sys.stdout, stderr=sys.stderr)
        else:
            reads = " ".join(args.input_file)
            subprocess.call(
                f'''
                cat {reads} > {args.output_dir}/{args.basename}.fq.gz
                ''', shell=True, stdout=sys.stdout, stderr=sys.stderr)


def main():
    parser = argparse.ArgumentParser("metapi misc")
    parser.add_argument("--basename", dest="basename")
    parser.add_argument("--input-file", dest="input_file", nargs="+")
    parser.add_argument("--output-dir", dest="output_dir")

    args = parser.parse_args()

    link_or_cat(args)


if __name__ == "__main__":
    main()



#!/usr/bin/env python3

import os
import concurrent.futures
import pandas as pd


def parse(stats_file):
    if os.path.exists(stats_file):
        try:
            df = pd.read_csv(stats_file, sep="\t")
        except pd.errors.EmptyDataError:
            print("%s is empty, please check" % stats_file)
            return None

        if not df.empty:
            return df
        else:
            return None
    else:
        print("%s is not exists" % stats_file)
        return None


def merge(input_list, func, workers, **kwargs):
    df_list = []
    with concurrent.futures.ProcessPoolExecutor(max_workers=workers) as executor:
        for df in executor.map(func, input_list):
            if df is not None:
                df_list.append(df)

    df_ = pd.concat(df_list)

    if "output" in kwargs:
        df_.to_csv(kwargs["output"], sep="\t", index=False)
    return df_


def merge2(input_list, func, workers, **kwargs):
    df1_list = []
    df2_list = []
    with concurrent.futures.ProcessPoolExecutor(max_workers=workers) as executor:
        for df1, df2 in executor.map(func, input_list):
            if df1 is not None:
                df1_list.append(df1)
            if df2 is not None:
                df2_list.append(df2)

    df_1 = pd.concat(df1_list)
    df_2 = pd.concat(df2_list)

    if "output_1" in kwargs:
        df_1.to_csv(kwargs["output_1"], sep="\t", index=False)
    if "output_2" in kwargs:
        df_2.to_csv(kwargs["output_2"], sep="\t", index=False)

    return df_1, df_2



#!/usr/bin/env python3

import os
import re
import math
import argparse
import subprocess

# use warnings.warn() rather than print() to output info in this script
# because snakemake expects the jobid to be the only output
import warnings

from snakemake import io
from snakemake.utils import read_job_properties

DEFAULT_JOB_NAME = "snakemake_job"
QSUB_DEFAULTS = "-cwd -V"
CLUSTER_CONFIG = "cluster.yaml"

# SGE syntax for options is `-option [value]` and for resources is `-l name=value`
# we therefore distinguish the two in this script to make it easier to handle.
# We also define some aliases for options and resources so that the rules can
# be more expressive than a list of cryptic SGE resources.

# We additionally pickup a list of environment modules which will be loaded in the
# jobscript

OPTION_MAPPING = {
    "binding": ("binding",),
    "cwd"    : ("cwd",),
    "e"      : ("e", "error"),
    "hard"   : ("hard",),
    "j"      : ("j", "join"),
    "m"      : ("m", "mail_options"),
    "M"      : ("M", "email"),
    "notify" : ("notify",),
    "now"    : ("now",),
    "N"      : ("N", "name"),
    "o"      : ("o", "output"),
    "P"      : ("P", "project"),
    "p"      : ("p", "priority"),
    "pe"     : ("pe", "parallel_environment"),
    "pty"    : ("pty",),
    "q"      : ("q", "queue"),
    "R"      : ("R", "reservation"),
    "r"      : ("r", "rerun"),
    "soft"   : ("soft",),
    "v"      : ("v", "variable"),
    "V"      : ("V", "export_env")
}

RESOURCE_MAPPING = {
    # default queue resources
    "qname"            : ("qname",),
    "hostname"         : ("hostname",),
    # "notify" -- conflicts with OPTION_MAPPING
    "calendar"         : ("calendar",),
    "min_cpu_interval" : ("min_cpu_interval",),
    "tmpdir"           : ("tmpdir",),
    "seq_no"           : ("seq_no",),
    "s_rt"             : ("s_rt", "soft_runtime", "soft_walltime"),
    "h_rt"             : ("h_rt", "time", "runtime", "walltime"),
    "s_cpu"            : ("s_cpu", "soft_cpu"),
    "h_cpu"            : ("h_cpu", "cpu"),
    "s_data"           : ("s_data", "soft_data"),
    "h_data"           : ("h_data", "data"),
    "s_stack"          : ("s_stack", "soft_stack"),
    "h_stack"          : ("h_stack", "stack"),           
    "s_core"           : ("s_core", "soft_core"),
    "h_core"           : ("h_core", "core"),
    "s_rss"            : ("s_rss", "soft_resident_set_size"),
    "h_rss"            : ("h_rss", "resident_set_size"),
    # default host resources
    "slots"            : ("slots",),
    "s_vmem"           : ("s_vmem", "soft_memory", "soft_virtual_memory"),
    "h_vmem"           : ("h_vmem", "mem", "memory", "virtual_memory"),
    "s_fsize"          : ("s_fsize", "soft_file_size"),
    "h_fsize"          : ("h_fsize", "file_size"),
}

def add_custom_resources(resources, resource_mapping=RESOURCE_MAPPING):
    """Adds new resources to resource_mapping.

       resources -> dict where key is sge resource name and value is a 
                    single name or a list of names to be used as aliased
    """
    for key, val in resources.items():
        if key not in resource_mapping:
            resource_mapping[key] = tuple()

        # make sure the resource name itself is an alias
        resource_mapping[key] += (key,)
        if isinstance(val, list):
            for alias in val:
                if val != key:
                    resource_mapping[key] += (alias,)
        else:
            if val != key:
                resource_mapping[key] += (val,)

def add_custom_options(options, option_mapping=OPTION_MAPPING):
    """Adds new options to option_mapping.

       options -> dict where key is sge option name and value is a single name
                  or a list of names to be used as aliased
    """
    for key, val in options.items():
        if key not in option_mapping:
            option_mapping[key] = tuple()

        # make sure the option name itself is an alias
        option_mapping[key] += (key,)
        if isinstance(val, list):
            for alias in val:
                if val != key:
                    option_mapping[key] += (alias,)
        else:
            if val != key:
                option_mapping[key] += (val,)

def parse_jobscript():
    """Minimal CLI to require/only accept single positional argument."""
    p = argparse.ArgumentParser(description="SGE snakemake submit script")
    p.add_argument("jobscript", help="Snakemake jobscript with job properties.")
    return p.parse_args().jobscript

def parse_qsub_defaults(parsed):
    """Unpack QSUB_DEFAULTS."""
    d = parsed.split() if type(parsed) == str else parsed
    
    options={}
    for arg in d:
        if "=" in arg:
            k,v = arg.split("=")
            options[k.strip("-")] = v.strip()
        else:
            options[arg.strip("-")] = ""
    return options

def format_job_properties(string):
    # we use 'rulename' rather than 'rule' for consistency with the --cluster-config 
    # snakemake option
    return string.format(rulename=job_properties['rule'], jobid=job_properties['jobid'])


def parse_qsub_settings(source, resource_mapping=RESOURCE_MAPPING, option_mapping=OPTION_MAPPING):
    job_options = { "options" : {}, "resources" : {}}

    for skey, sval in source.items():
        found = False
        for rkey, rval in resource_mapping.items():
            if skey in rval:
                found = True
                # Snakemake resources can only be defined as integers, but SGE interprets
                # plain integers for memory as bytes. This hack means we interpret memory
                # requests as gigabytes
                if (rkey == 's_vmem') or (rkey == 'h_vmem'):
                    job_options["resources"].update({rkey : str(sval) + 'G'})
                else:
                    job_options["resources"].update({rkey : sval})
                break
        if found: continue
        for okey, oval in option_mapping.items():
            if skey in oval:
                found = True
                job_options["options"].update({okey : sval})
                break
        if not found:
            raise KeyError(f"Unknown SGE option or resource: {skey}")

    return job_options

def load_cluster_config(path):
    """Load config to dict either from absolute path or relative to profile dir."""
    if path:
        path = os.path.join(os.path.dirname(__file__), os.path.expandvars(path))
        default_cluster_config = io.load_configfile(path)
    else:
        default_cluster_config = {}
    if "__default__" not in default_cluster_config:
        default_cluster_config["__default__"] = {}
    return default_cluster_config

def ensure_directory_exists(path):
    """Check if directory exists and create if not"""
    directory = os.path.dirname(path)
    if not os.path.exists(directory):
        os.makedirs(directory, exist_ok=True)
    return


def update_double_dict(outer, inner):
    """Similar to dict.update() but does the update on nested dictionaries"""
    for k, v in outer.items():
        outer[k].update(inner[k])

def sge_option_string(key, val):
    if val == "":
        return f"-{key}"
    if type(val) == bool:
        return f"-{key} " + ("yes" if val else "no")
    return format_job_properties(f"-{key} {val}")

def sge_resource_string(key, val):
    if val == "":
        return f"-l {key}"
    if type(val) == bool:
        return f"-{key}=" + ("true" if val else "false")
    return f"-l {key}={val}"

def submit_job(jobscript, qsub_settings):
    """Submit jobscript and return jobid."""
    flatten = lambda l: [item for sublist in l for item in sublist]
    batch_options = flatten([sge_option_string(k,v).split() for k, v in qsub_settings["options"].items()])
    batch_resources = flatten([sge_resource_string(k, v).split() for k, v in qsub_settings["resources"].items()])
    try:
        # -terse means only the jobid is returned rather than the normal 'Your job...' string
        jobid = subprocess.check_output(["qsub", "-terse"] + batch_options + batch_resources + [jobscript]).decode().rstrip()
    except subprocess.CalledProcessError as e:
        raise e
    except Exception as e:
        raise e
    return jobid

qsub_settings = { "options" : {}, "resources" : {}}

jobscript = parse_jobscript()

# get the job properties dictionary from snakemake 
job_properties = read_job_properties(jobscript)

# load the default cluster config
cluster_config = load_cluster_config(CLUSTER_CONFIG)

add_custom_resources(cluster_config["__resources__"])

add_custom_options(cluster_config["__options__"])

# qsub default arguments
update_double_dict(qsub_settings, parse_qsub_settings(parse_qsub_defaults(QSUB_DEFAULTS)))

# cluster_config defaults
update_double_dict(qsub_settings, parse_qsub_settings(cluster_config["__default__"]))

# resources defined in the snakemake file (note that these must be integer)
# we pass an empty dictionary for option_mapping because options should not be
# specified in the snakemake file
update_double_dict(qsub_settings, parse_qsub_settings(job_properties.get("resources", {}), option_mapping={}))

# get any rule specific options/resources from the default cluster config
update_double_dict(qsub_settings, parse_qsub_settings(cluster_config.get(job_properties.get("rule"), {})))

# get any options/resources specified through the --cluster-config command line argument
update_double_dict(qsub_settings, parse_qsub_settings(job_properties.get("cluster", {})))

# ensure qsub output dirs exist
for o in ("o", "e"):
    ensure_directory_exists(qsub_settings["options"][o]) if o in qsub_settings["options"] else None

# submit job and echo id back to Snakemake (must be the only stdout)
print(submit_job(jobscript, qsub_settings))




#!/usr/bin/env python

from metapi.configer import metaconfig
from metapi.configer import parse_yaml
from metapi.configer import update_config
from metapi.configer import custom_help_formatter

from metapi.tooler import parse
from metapi.tooler import merge

from metapi.simulator import parse_genomes
from metapi.simulator import get_simulate_info
from metapi.simulator import simulate_short_reads

from metapi.sampler import parse_samples
from metapi.sampler import get_reads
from metapi.sampler import get_sample_id
from metapi.sampler import get_sample_id_
from metapi.sampler import get_samples_id_by_assembly_group
from metapi.sampler import get_samples_id_by_binning_group
from metapi.sampler import get_assembly_group_by_binning_group
from metapi.sampler import get_binning_group_by_assembly_group
from metapi.sampler import get_multibinning_group_by_assembly_group

from metapi.qcer import change
from metapi.qcer import compute_host_rate
from metapi.qcer import qc_bar_plot
from metapi.qcer import parse_fastp_json

from metapi.assembler import assembler_init
from metapi.assembler import parse_assembly

from metapi.aligner import flagstats_summary

from metapi.predictor import parse_gff
from metapi.predictor import extract_faa

from metapi.binner import get_binning_info
from metapi.binner import generate_bins
from metapi.binner import extract_bins_report
from metapi.binner import combine_jgi

from metapi.checkmer import checkm_prepare
from metapi.checkmer import checkm_reporter

from metapi.classifier import demultiplex
from metapi.classifier import gtdbtk_prepare

from metapi.uploader import gen_samples_info
from metapi.uploader import gen_info

from metapi.__about__ import __version__, __author__

name = "metapi"



#!/usr/bin/env python

import os
import sys
import glob
import subprocess


bins_prefix = sys.argv[1].replace("dastools.bin", "")

bins_list = glob.glob(os.path.join(sys.argv[1] + "_DASTool_bins", "*.fa"))

if len(bins_list) > 0:
    for bin_fa in bins_list:
        if (os.path.getsize(bin_fa) > 0) and  (not "*" in bin_fa):
            binner = os.path.basename(bin_fa).split(".")[0]
            if (binner != "unbinned") and (binner != "*"):
                bin_fa_ = bins_prefix + os.path.basename(bin_fa).replace(binner, binner +"_dastools")
                subprocess.run(f'''mv {bin_fa} {bin_fa_}''', shell=True)


#!/usr/bin/env python3


import sys, os
from subprocess import Popen, PIPE
import yaml


def eprint(*args, **kwargs):
    print(*args, file=sys.stderr, **kwargs)


# let snakemake read job_properties
from snakemake.utils import read_job_properties



jobscript = sys.argv[1]
job_properties = read_job_properties(jobscript)

#default paramters defined in cluster_spec (accessed via snakemake read_job_properties)
cluster_param= job_properties["cluster"]

if job_properties["type"]=='single':
    cluster_param['name'] = job_properties['rule']
elif job_properties["type"]=='group':
    cluster_param['name'] = job_properties['groupid']
else:
    raise NotImplementedError(f"Don't know what to do with job_properties['type']=={job_properties['type']}")


# don't overwrite default parameters if defined in rule (or config file)
if ('threads' in job_properties) and ('threads' not in cluster_param):
    cluster_param["threads"] = job_properties["threads"]
for res in ['time','mem']:
    if (res in job_properties["resources"]) and (res not in cluster_param):
        cluster_param[res] = job_properties["resources"][res]

# time in hours
if "time" in cluster_param:
    cluster_param["time"]=int(cluster_param["time"]*60)


# check which system you are on and load command command_options
key_mapping_file=os.path.join(os.path.dirname(__file__),"key_mapping.yaml")
command_options=yaml.load(open(key_mapping_file),
                          Loader=yaml.BaseLoader)
system= command_options['system']
command= command_options[system]['command']

key_mapping= command_options[system]['key_mapping']

# construct command:
for  key in key_mapping:
    if key in cluster_param:
        command+=" "
        command+=key_mapping[key].format(cluster_param[key])

command+=' {}'.format(jobscript)

eprint("submit command: "+command)

p = Popen(command.split(' '), stdout=PIPE, stderr=PIPE)
output, error = p.communicate()
if p.returncode != 0:
    raise Exception("Job can't be submitted\n"+output.decode("utf-8")+error.decode("utf-8"))
else:
    res= output.decode("utf-8")

    if system=='lsf':
        import re
        match = re.search(r"Job <(\d+)> is submitted", res)
        jobid = match.group(1)

    elif system=='pbs':
        jobid= res.strip().split('.')[0]

    else:
        jobid= int(res.strip().split()[-1])

    print(jobid)



#!/usr/bin/env python

import argparse
import os
import subprocess
import sys
import textwrap
from io import StringIO

import pandas as pd

import metapi

WORKFLOWS_MAG = [
    "simulate_all",
    "prepare_short_reads_all",
    "prepare_long_reads_all",
    "prepare_reads_all",
    "raw_fastqc_all",
    "raw_report_all",
    "raw_all",
    "trimming_sickle_all",
    "trimming_fastp_all",
    "trimming_report_all",
    "trimming_all",
    "rmhost_bwa_all",
    "rmhost_bowtie2_all",
    "rmhost_minimap2_all",
    "rmhost_kraken2_all",
    "rmhost_kneaddata_all",
    "rmhost_report_all",
    "rmhost_all",
    "qcreport_all",
    "assembly_megahit_all",
    "assembly_idba_ud_all",
    "assembly_metaspades_all",
    "assembly_spades_all",
    "assembly_plass_all",
    "assembly_opera_ms_all",
    "assembly_metaquast_all",
    "assembly_report_all",
    "assembly_all",
    "alignment_base_depth_all",
    "alignment_report_all",
    "alignment_all",
    "binning_metabat2_coverage_all",
    "binning_metabat2_all",
    "binning_maxbin2_all",
    "binning_concoct_all",
    "binning_graphbin2_all",
    "binning_dastools_all",
    "binning_vamb_prepare_all",
    "binning_vamb_all",
    "binning_report_all",
    "binning_all",
    "identify_virsorter2_all",
    "identify_deepvirfinder_all",
    "identify_single_all",
    "identify_phamb_all",
    "identify_multi_all",
    "identify_all",
    "predict_scaftigs_gene_prodigal_all",
    "predict_scaftigs_gene_prokka_all",
    "predict_bins_gene_prodigal_all",
    "predict_bins_gene_prokka_all",
    "predict_scaftigs_gene_all",
    "predict_bins_gene_all",
    "predict_all",
    "checkm_all",
    "dereplicate_mags_drep_all",
    "dereplicate_mags_all",
    "taxonomic_gtdbtk_all",
    "taxonomic_all",
    "upload_sequencing_all",
    "upload_assembly_all",
    "upload_all",
    "all",
]

WORKFLOWS_GENE = [
    "simulate_all",
    "prepare_reads_all",
    "raw_fastqc_all",
    "raw_report_all",
    "raw_all",
    "trimming_sickle_all",
    "trimming_fastp_all",
    "trimming_report_all",
    "trimming_all",
    "rmhost_bwa_all",
    "rmhost_bowtie2_all",
    "rmhost_minimap2_all",
    "rmhost_report_all",
    "rmhost_all",
    "qcreport_all",
    "assebmly_megahit_all",
    "assembly_idba_ud_all",
    "assembly_metaspades_all",
    "assembly_spades_all",
    "assembly_plass_all",
    "assembly_metaquast_all",
    "assembly_report_all",
    "predict_scaftigs_gene_prodigal_all",
    "predict_scaftigs_gene_prokka_all",
    "predict_scafitgs_gene_all",
    "predict_all",
    "dereplicate_gene_cdhit_all",
    "dereplicate_gene_all",
    "upload_sequencing_all",
    "upload_assembly_all",
    "upload_all",
    "all",
]


def run_snakemake(args, unknown, snakefile, workflow):
    conf = metapi.parse_yaml(args.config)

    if not os.path.exists(conf["params"]["samples"]):
        print("Please specific samples list on init step or change config.yaml manualy")
        sys.exit(1)

    cmd = [
        "snakemake",
        "--snakefile",
        snakefile,
        "--configfile",
        args.config,
        "--cores",
        str(args.cores),
        "--until",
        args.task
    ] + unknown

    if "--touch" in unknown:
        pass
    elif args.conda_create_envs_only:
        cmd += ["--use-conda", "--conda-create-envs-only"]
        if args.conda_prefix is not None:
            cmd += ["--conda-prefix", args.conda_prefix]
    else:
        cmd += [
            "--rerun-incomplete",
            "--keep-going",
            "--printshellcmds",
            "--reason",
        ]

        if args.use_conda:
            cmd += ["--use-conda"]
            if args.conda_prefix is not None:
                cmd += ["--conda-prefix", args.conda_prefix]

        if args.list:
            cmd += ["--list"]
        elif args.run_local:
            cmd += ["--local-cores", str(args.local_cores),
                    "--jobs", str(args.jobs)]
        elif args.run_remote:
            cmd += ["--profile", args.profile,
                    "--local-cores", str(args.local_cores),
                    "--jobs", str(args.jobs)]
        elif args.debug:
            cmd += ["--debug-dag"]
        else: 
            cmd += ["--dry-run"]

        if args.dry_run and ("--dry-run" not in cmd):
            cmd += ["--dry-run"]
 
    cmd_str = " ".join(cmd).strip()
    print("Running metapi %s:\n%s" % (workflow, cmd_str))

    env = os.environ.copy()
    proc = subprocess.Popen(
        cmd_str,
        shell=True,
        stdout=sys.stdout,
        stderr=sys.stderr,
        env=env,
    )
    proc.communicate()

    print(f'''\nReal running cmd:\n{cmd_str}''')


def update_config_tools(conf, begin, trimmer, rmhoster, assemblers, binners):
    conf["params"]["simulate"]["do"] = False
    conf["params"]["begin"] = begin

    for trimmer_ in ["sickle", "fastp"]:
        if trimmer_ == trimmer:
            conf["params"]["trimming"][trimmer_]["do"] = True
        else:
            conf["params"]["trimming"][trimmer_]["do"] = False

    for rmhoster_ in ["bwa", "bowtie2", "minimap2", "kraken2", "kneaddata"]:
        if rmhoster_ == rmhoster:
            conf["params"]["rmhost"][rmhoster_]["do"] = True
        else:
            conf["params"]["rmhost"][rmhoster_]["do"] = False

    for assembler_ in ["megahit", "idba_ud", "metaspades", "spades", "plass", "opera_ms"]:
        if assembler_ in assemblers:
            conf["params"]["assembly"][assembler_]["do"] = True
        else:
            conf["params"]["assembly"][assembler_]["do"] = False

    for binner_ in ["metabat2", "maxbin2", "concoct", "graphbin2", "vamb", "dastools"]:
        if binner_ in binners:
            conf["params"]["binning"][binner_]["do"] = True
        else:
            conf["params"]["binning"][binner_]["do"] = False

    if begin == "simulate":
        conf["params"]["simulate"]["do"] = True
    elif begin == "rmhost":
        conf["params"]["trimming"][trimmer]["do"] = False
    elif (begin == "assembly") or (begin == "binning"):
        conf["params"]["raw"]["save_reads"] = True
        conf["params"]["raw"]["fastqc"]["do"] = False
        conf["params"]["qcreport"]["do"] = False

        conf["params"]["trimming"][trimmer]["do"] = False
        conf["params"]["rmhost"][rmhoster]["do"] = False
    return conf


def init(args, unknown):
    if args.workdir:
        project = metapi.metaconfig(args.workdir)
        print(project.__str__())
        project.create_dirs()
        conf = project.get_config()

        for env_name in conf["envs"]:
            conf["envs"][env_name] = os.path.join(os.path.realpath(args.workdir), f"envs/{env_name}.yaml")

        conf = update_config_tools(
            conf, args.begin, args.trimmer, args.rmhoster, args.assembler, args.binner
        )

        if args.samples:
            conf["params"]["samples"] = args.samples
        else:
            print("Please supply samples table")
            sys.exit(-1)

        metapi.update_config(
            project.config_file, project.new_config_file, conf, remove=False
        )
    else:
        print("Please supply a workdir!")
        sys.exit(-1)


def mag_wf(args, unknown):
    snakefile = os.path.join(os.path.dirname(__file__), "snakefiles/mag_wf.smk")
    run_snakemake(args, unknown, snakefile, "mag_wf")


def gene_wf(args, unknown):
    snakefile = os.path.join(os.path.dirname(__file__), "snakefiles/gene_wf.smk")
    run_snakemake(args, unknown, snakefile, "gene_wf")


def snakemake_summary(snakefile, configfile, task):
    cmd = [
        "snakemake",
        "--snakefile",
        snakefile,
        "--configfile",
        configfile,
        "--until",
        task,
        "--summary",
    ]
    cmd_out = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    summary = pd.read_csv(StringIO(cmd_out.stdout.read().decode()), sep="\t")
    return summary


def sync(args, unknown):
    snakefile = os.path.join(
        os.path.dirname(__file__), f"snakefiles/{args.workflow}.smk"
    )
    summary_df = snakemake_summary(snakefile, args.config, args.task)
    conf = metapi.parse_yaml(args.config)

    if conf["params"]["simulate"]["do"]:
        samples_df = metapi.parse_genomes(
            conf["params"]["samples"], conf["output"]["simulate"], args.check_samples
        )
    else:
        samples_df = metapi.parse_samples(
            conf["params"]["samples"],
            conf["params"]["interleaved"],
            conf["params"]["reads_layout"],
            conf["params"]["begin"],
            args.check_samples,
        )

    samples_index = samples_df.index.unique()
    count = -1
    for i in range(0, len(samples_index), args.split_num):
        count += 1
        outdir = os.path.abspath(os.path.join(args.outdir, args.name + f"_{count}"))
        os.makedirs(outdir, exist_ok=True)
        samples_file = os.path.join(outdir, f"samples_{count}.tsv")
        config_file = os.path.join(outdir, "config.yaml")

        samples = samples_df.loc[
            samples_index[i : i + args.split_num],
        ]
        samples.to_csv(samples_file, sep="\t", index=False)
        conf["params"]["samples"] = samples_file
        metapi.update_config(args.config, config_file, conf, remove=False)

        summary = snakemake_summary(snakefile, config_file, args.task)
        summary_ = summary_df[summary_df.output_file.isin(summary.output_file)]
        sync_sh = os.path.join(
            args.workdir, f"sync-{args.workflow}-{args.task}_{count}.sh"
        )
        with open(sync_sh, "w") as oh:
            print(f"generating {sync_sh}")
            for output_file in summary_["output_file"]:
                if output_file != "-" and (not pd.isna(output_file)):
                    output_file_path = os.path.join(args.workdir, output_file)
                    oh.write(
                        f"rsync --archive --relative --progress {output_file_path} {outdir}\n"
                    )
            for log_file in summary_["log-file(s)"]:
                if log_file != "-" and (not pd.isna(log_file)):
                    log_file_path = os.path.join(args.workdir, log_file)
                    oh.write(
                        f"rsync --archive --relative --progress {log_file_path} {outdir}\n"
                    )

    print(f"please change current directory to {args.workdir} to run sync script")


def main():
    banner = """

  .___  ___.  _______ .___________.    ___      .______    __
  |   \/   | |   ____||           |   /   \     |   _  \  |  |
  |  \  /  | |  |__   `---|  |----`  /  ^  \    |  |_)  | |  |
  |  |\/|  | |   __|      |  |      /  /_\  \   |   ___/  |  |
  |  |  |  | |  |____     |  |     /  _____  \  |  |      |  |
  |__|  |__| |_______|    |__|    /__/     \__\ | _|      |__|

            Omics for All, Open Source for All

 A general metagenomics data mining system focus on robust microbiome research

"""

    parser = argparse.ArgumentParser(
        formatter_class=argparse.RawDescriptionHelpFormatter,
        description=textwrap.dedent(banner),
        prog="metapi",
    )
    parser.add_argument(
        "-v",
        "--version",
        action="store_true",
        default=False,
        help="print software version and exit",
    )

    common_parser = argparse.ArgumentParser(add_help=False)
    common_parser.add_argument(
        "-d",
        "--workdir",
        metavar="WORKDIR",
        type=str,
        default="./",
        help="project workdir",
    )
    common_parser.add_argument(
        "--check-samples",
        dest="check_samples",
        default=False,
        action="store_true",
        help="check samples, default: False",
    )

    run_parser = argparse.ArgumentParser(add_help=False)
    run_parser.add_argument(
        "--config",
        type=str,
        default="./config.yaml",
        help="config.yaml",
    )
    run_parser.add_argument(
        "--profile",
        type=str,
        default="./profiles/slurm",
        help="cluster profile name",
    )
    run_parser.add_argument(
        "--cores",
        type=int,
        default=240,
        help="all job cores, available on '--run-local'")
    run_parser.add_argument(
        "--local-cores",
        type=int,
        dest="local_cores",
        default=8,
        help="local job cores, available on '--run-remote'")
    run_parser.add_argument(
        "--jobs",
        type=int,
        default=30,
        help="cluster job numbers, available on '--run-remote'")
    run_parser.add_argument(
        "--list",
        default=False,
        action="store_true",
        help="list pipeline rules",
    )
    run_parser.add_argument(
        "--debug",
        default=False,
        action="store_true",
        help="debug pipeline",
    )
    run_parser.add_argument(
        "--dry-run",
        default=False,
        dest="dry_run",
        action="store_true",
        help="dry run pipeline",
    )
    run_parser.add_argument(
        "--run-local",
        default=False,
        action="store_true",
        help="run pipeline on local computer",
    )
    run_parser.add_argument(
        "--run-remote",
        default=False,
        action="store_true",
        help="run pipeline on remote cluster",
    )
    run_parser.add_argument(
        "--cluster-engine",
        default="slurm",
        choices=["slurm", "sge", "lsf", "pbs-torque"],
        help="cluster workflow manager engine, support slurm(sbatch) and sge(qsub)"
    )
    run_parser.add_argument("--wait", type=int, default=60, help="wait given seconds")
    run_parser.add_argument(
        "--use-conda",
        default=False,
        dest="use_conda",
        action="store_true",
        help="use conda environment",
    )
    run_parser.add_argument(
        "--conda-prefix",
        default="~/.conda/envs",
        dest="conda_prefix",
        help="conda environment prefix",
    )
    run_parser.add_argument(
        "--conda-create-envs-only",
        default=False,
        dest="conda_create_envs_only",
        action="store_true",
        help="conda create environments only",
    )

    subparsers = parser.add_subparsers(title="available subcommands", metavar="")
    parser_init = subparsers.add_parser(
        "init",
        formatter_class=metapi.custom_help_formatter,
        parents=[common_parser],
        prog="metapi init",
        help="init project",
    )
    parser_mag_wf = subparsers.add_parser(
        "mag_wf",
        formatter_class=metapi.custom_help_formatter,
        parents=[common_parser, run_parser],
        prog="metapi mag_wf",
        help="metagenome-assembly-genome pipeline",
    )
    parser_gene_wf = subparsers.add_parser(
        "gene_wf",
        formatter_class=metapi.custom_help_formatter,
        parents=[common_parser, run_parser],
        prog="metapi gene_wf",
        help="metagenome-assembly-gene pipeline",
    )

    parser_sync = subparsers.add_parser(
        "sync",
        formatter_class=metapi.custom_help_formatter,
        parents=[common_parser],
        prog="metapi sync",
        help="metapi sync project",
    )

    parser_init.add_argument(
        "-s",
        "--samples",
        type=str,
        default=None,
        help="""desired input:
samples list, tsv format required.

if begin from trimming, rmhost, or assembly:
    if it is fastq:
        the header is: [sample_id, assembly_group, binning_group, fq1, fq2]
    if it is sra:
        the header is: [sample_id, assembly_group, binning_group, sra]

if begin from simulate:
        the header is: [id, genome, abundance, reads_num, model]

""",
    )
    parser_init.add_argument(
        "-b",
        "--begin",
        type=str,
        default="trimming",
        choices=["simulate", "trimming", "rmhost", "assembly", "binning", "checkm"],
        help="pipeline starting point",
    )
    parser_init.add_argument(
        "--trimmer",
        type=str,
        default="fastp",
        required=False,
        choices=["sickle", "fastp"],
        help="which trimmer used",
    )
    parser_init.add_argument(
        "--rmhoster",
        type=str,
        default="bowtie2",
        required=False,
        choices=["bwa", "bowtie2", "minimap2", "kraken2", "kneaddata"],
        help="which rmhoster used",
    )
    parser_init.add_argument(
        "--assembler",
        nargs="+",
        default=["metaspades"],
        required=False,
        choices=["idba-ud", "megahit", "metaspades", "spades", "opera-ms"],
        help="which assembler used, required when begin with binning, can be changed in config.yaml",
    )
    parser_init.add_argument(
        "--binner",
        nargs="+",
        required=False,
        default=["metabat2", "concoct", "maxbin2", "vamb", "dastools"],
        help="wchich binner used",
    )
    parser_init.set_defaults(func=init)

    parser_mag_wf.add_argument(
        "task",
        metavar="TASK",
        nargs="?",
        type=str,
        default="all",
        choices=WORKFLOWS_MAG,
        help="pipeline end point. Allowed values are " + ", ".join(WORKFLOWS_MAG),
    )
    parser_mag_wf.set_defaults(func=mag_wf)

    parser_gene_wf.add_argument(
        "task",
        metavar="TASK",
        nargs="?",
        type=str,
        default="all",
        choices=WORKFLOWS_GENE,
        help="pipeline end point. Allowed values are " + ", ".join(WORKFLOWS_GENE),
    )
    parser_gene_wf.set_defaults(func=gene_wf)

    parser_sync.add_argument(
        "workflow",
        metavar="WORKFLOW",
        nargs="?",
        type=str,
        default="mag_wf",
        choices=["mag_wf", "gene_wf"],
        help="workflow. Allowed values are mag_wf, gene_wf",
    )
    parser_sync.add_argument(
        "task", 
        metavar="TASK",
        nargs="?",
        type=str,
        default="all",
        help="pipeline end point",
    )
    parser_sync.add_argument(
        "--config",
        type=str,
        default="./config.yaml",
        help="config.yaml",
    )
    parser_sync.add_argument("--name", type=str, required=True, help="project basename")
    parser_sync.add_argument(
        "--outdir", type=str, required=True, help="sync to a directory"
    )
    parser_sync.add_argument(
        "--split-num",
        type=int,
        default=1,
        dest="split_num",
        help="split project to sync directory",
    )
    parser_sync.set_defaults(func=sync)

    args, unknown = parser.parse_known_args()

    try:
        if args.version:
            print("metapi version %s" % metapi.__version__)
            sys.exit(0)
        args.func(args, unknown)
    except AttributeError as e:
        print(e)
        parser.print_help()


if __name__ == "__main__":
    main()



#!/usr/bin/env python

import re
import numpy as np
from metapi import tooler


def assembler_init(contigs_length_range, groups):
    global CONTIGS_LENGTH_RANGES__
    global GROUP_BY_

    temp = sorted(list(set(contigs_length_range)))
    if (temp is None) or (temp == [0]) or (len(temp) == 0):
        CONTIGS_LENGTH_RANGES__ = [(0, 1500), (1500, 2000), (2000, 2500), 2500]
    elif temp[0] == 0:
        CONTIGS_LENGTH_RANGES__ = [(i, j) for i, j in zip(temp[0:-1], temp[1:])] + [
            temp[-1]
        ]
    else:
        CONTIGS_LENGTH_RANGES__ = [(i, j) for i, j in zip([0] + temp[0:-1], temp)] + [
            temp[-1]
        ]

    GROUP_BY_ = groups


def Nx(x):
    Nx_len = np.zeros(21)
    Nx_rate = np.linspace(0, 1, 21)
    Nx_len_ = Nx_rate * sum(x)

    len_sum = 0
    for i in sorted(x)[::-1]:
        len_sum += i
        for j in range(0, len(Nx_len)):
            if (Nx_len[j] == 0.0) and (len_sum >= Nx_len_[j]):
                Nx_len[j] = i
    return list(Nx_len)


def GC_content(x):
    return (x[("#G", "sum")] + x[("#C", "sum")]) / x[("length", "sum")]


def cal_len_range(x):
    len_dict = {i: 0 for i in CONTIGS_LENGTH_RANGES__}
    for seq_len in x:
        if seq_len >= CONTIGS_LENGTH_RANGES__[-1]:
            len_dict[CONTIGS_LENGTH_RANGES__[-1]] += 1
        else:
            for i in CONTIGS_LENGTH_RANGES__[:-1]:
                if i[0] <= seq_len < i[1]:
                    len_dict[i] += 1
                    break
    return tuple([len_dict[i] for i in CONTIGS_LENGTH_RANGES__])


def cumulative_len(x):
    """
    for x, y in enumerate(len_y):
        plt.plot(x, y)
    """
    len_y = [0]
    for l in sorted(x)[::-1]:
        len_y.append(len_y[-1] + l)
    return len_y


def parse_assembly(input_tuple):
    df_ = tooler.parse(input_tuple[0])

    if (df_ is not None) and (not df_.empty):
        df_ = df_.query('length >= %d' % input_tuple[1])

        df = (
            df_.groupby(GROUP_BY_)
            .agg(
                {
                    "chr": ["count"],
                    "length": ["sum", "min", "max", "std", Nx, cal_len_range],
                    "#A": ["sum", "min", "max", "std"],
                    "#C": ["sum", "min", "max", "std"],
                    "#G": ["sum", "min", "max", "std"],
                    "#T": ["sum", "min", "max", "std"],
                    "#CpG": ["sum", "min", "max", "std"],
                }
            )
            .reset_index()
        )
        df["GC_content"] = df.apply(lambda x: GC_content(x), axis=1)

        N_ = []
        LEN_LIST = []

        for N in range(0, 105, 5):
            N_.append("N" + str(N))

        for i in range(0, 21):
            df[("length", N_[i])] = df.apply(lambda x: x[("length", "Nx")][i], axis=1)

        for i in range(0, len(CONTIGS_LENGTH_RANGES__) - 1):
            len_tuple = CONTIGS_LENGTH_RANGES__[i]
            LEN_LIST.append(len_tuple[0])
            LEN_LIST.append(len_tuple[1])
            len_range_str = "[%d, %d)" % (len_tuple[0], len_tuple[1])
            df[("length", len_range_str)] = df.apply(
                lambda x: x[("length", "cal_len_range")][i], axis=1
            )

        LEN_LIST.append(CONTIGS_LENGTH_RANGES__[-1])

        df[("length", "[%d, )" % CONTIGS_LENGTH_RANGES__[-1])] = df.apply(
            lambda x: x[("length", "cal_len_range")][-1], axis=1
        )

        LEN_LIST = sorted(list(set(LEN_LIST)))
        for i in range(0, len(LEN_LIST)):
            df[("length", ">=%d" % LEN_LIST[i])] = df.apply(
                lambda x: sum(x[("length", "cal_len_range")][i:]), axis=1
            )
        return df
    else:
        return None



import subprocess
import sys
import time
import uuid
from pathlib import Path
from typing import Tuple, List

if not __name__.startswith("tests.src."):
    sys.path.append(str(Path(__file__).parent.absolute()))
    from CookieCutter import CookieCutter
else:
    from .CookieCutter import CookieCutter

stdout = str
stderr = str


class TailError(Exception):
    pass


class OSLayer:
    """
    This class provides an abstract layer to communicating with the OS.
    Its main purpose is to enable OS operations mocking, so we don't actually need to
    make file operations or create processes.
    """

    @staticmethod
    def mkdir(directory: Path):
        directory.mkdir(parents=True, exist_ok=True)

    @staticmethod
    def remove_file(file: Path):
        if file.is_file():
            file.unlink()

    @staticmethod
    def run_process(cmd: str) -> Tuple[stdout, stderr]:
        completed_process = subprocess.run(
            cmd, check=True, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE
        )
        return (
            completed_process.stdout.decode().strip(),
            completed_process.stderr.decode().strip(),
        )

    @staticmethod
    def print(string: str):
        print(string)

    @staticmethod
    def get_uuid4_string() -> str:
        return str(uuid.uuid4())

    @staticmethod
    def tail(path: str, num_lines: int = 10) -> List[bytes]:
        if not Path(path).exists():
            # allow for filesystem latency
            time.sleep(CookieCutter.get_latency_wait())
            if not Path(path).exists():
                raise FileNotFoundError("{} does not exist.".format(path))

        process = subprocess.Popen(
            ["tail", "-n", str(num_lines), path],
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
        )
        exit_code = process.wait()
        if exit_code != 0:
            raise TailError(
                "Failed to execute the tail command on the file {} due to the "
                "following error:\n{}".format(path, process.stderr.read().decode())
            )
        return process.stdout.readlines()



#!/usr/bin/env python
import glob
import gzip
import os
import sys
import time

import pandas as pd
from Bio import bgzf


def parse_samples(
    samples_tsv,
    interleaved="false",
    reads_layout="pe",
    begin_point="trimming",
    check_samples=False,
):
    samples_df = pd.read_csv(
        samples_tsv, sep="\t", dtype={
            "sample_id": str,
            "fq1": str, "fq2": str,
            "assembly_group": str,
            "binning_group": str})
    samples_df["multibinning_group"] = samples_df["assembly_group"] + "__" + samples_df["binning_group"]
    samples_df = samples_df.set_index(["sample_id", "assembly_group", "binning_group", "multibinning_group"])

    cancel = False
    if "fq1" in samples_df.columns:
        for sample_id in samples_df.index.get_level_values("sample_id").unique():
            sample_id = str(sample_id)
            if "." in sample_id:
                print(f"{sample_id} contain '.', please remove '.', now quiting :)")
                cancel = True

            fq1_list = samples_df.loc[sample_id, :, :, :]["fq1"].dropna().tolist()
            fq2_list = samples_df.loc[sample_id, :, :, :]["fq2"].dropna().tolist()
            for fq_file in fq1_list:
                if not fq_file.endswith(".gz"):
                    print(f"{fq_file} need gzip format")
                    cancel = True
                if check_samples:
                    if not os.path.exists(fq_file):
                        print(f"{fq_file} not exists")
                        cancel = True
                    if (reads_layout == "pe") and (not interleaved):
                        if len(fq2_list) == 0:
                            print(f"{sample_id} fq2 not exists")
                            cancel = True
    elif "sra" in samples_df.columns:
        for sample_id in samples_df.index.get_level_values("sample_id").unique():
            sample_id = str(sample_id)
            if "." in sample_id:
                print(f"{sample_id} contain '.', please remove '.', now quiting :)")
                cancel = True

            if check_samples:
                sra_list = samples_df.loc[sample_id, :, :, :]["sra"].dropna().tolist()
                for sra_file in sra_list:
                    if not os.path.exists(sra_file):
                        print(f"{sra_file} not exists")
                        cancel = True
    else:
        print("wrong header: {header}".format(header=samples_df.columns))
        cancel = True

    if begin_point == "binning":
        if len(samples_df) != len(samples_df.index.unique()):
            print(
                "when begin with binning, samples id need to be unique, because we can't merge assembly"
            )
            cancel = True

        if check_samples:
            if "scaftigs" in samples_df.columns:
                for sample_id in samples_df.index.unique():
                    sample_id = str(sample_id)
                    scaftigs = samples_df.loc[sample_id, "scaftigs"]
                    if not os.path.exists(scaftigs):
                        print(f"{scaftigs} not exists")
                        cancel = True

    if cancel:
        sys.exit(-1)
    else:
        return samples_df


def parse_bins(bins_dir):
    bin_list = []
    for bin_ in glob.glob(bins_dir + "/*/*.fa"):
        bin_dict = dict()
        bin_dict["path"] = bin_.strip()
        bin_dict["id"] = os.path.basename(bin_).rstrip(".fa")
        bin_list.append(bin_dict)
    bins = pd.DataFrame(bin_list).set_index("id", drop=False)
    return bins


def get_reads(sample_df, wildcards, col):
    return sample_df.loc[wildcards.sample, :, :, :][col].dropna().tolist()


def get_samples_id_by_assembly_group(sample_df, assembly_group):
    return sample_df.loc[:, assembly_group, :, :].index.get_level_values("sample_id").unique()


def get_samples_id_by_binning_group(sample_df, binning_group):
    return sample_df.loc[:, :, binning_group, :].index.get_level_values("sample_id").unique()


def get_assembly_group_by_binning_group(sample_df, binning_group):
    return sample_df.loc[:, :, binning_group, :].index.get_level_values("assembly_group").unique()


def get_binning_group_by_assembly_group(sample_df, assembly_group):
    return sample_df.loc[:, assembly_group, :, :].index.get_level_values("binning_group").unique()


def get_multibinning_group_by_assembly_group(sample_df, assembly_group):
    return sample_df.loc[:, assembly_group, :, :].index.get_level_values("multibinning_group").unique()


def get_sample_id(sample_df, wildcards, col):
    return sample_df.loc[wildcards.sample, :, :][col].dropna()[0]


def get_sample_id_(sample_df, wildcards, col):
    return sample_df.loc[wildcards.sample_, :, :][col].dropna()[0]


def get_bin_id(bin_df, wildcards, col):
    return bin_df.loc[wildcards.bin, [col]].dropna()[0]


def sample_cluster_info(sample_df):
    '''
    vamb cluster
    '''
    binning_groups = sample_df.index.get_level_values("binning_group").unique()

    for binning_group in binning_groups:
        assembly_groups = sample_df.loc[:, :, binning_group].index.get_level_values("assembly_group").unique()
        assembly_groups = sorted(assembly_groups)
        for i in range(0, len(assembly_groups)):
            print(f'''{assembly_groups[i]}\tS{i+1}''')


#!/usr/bin/env/python
import os
import pandas as pd


# parse NCBI assembly report, include genbank and refseq, and check the path exist on BGI cluster

def check_ncbi_assembly_report_on_bgi(asm_summary, database, domain, check=True, low_memory=False):
    asm_summary = pd.read_csv(asm_summary, sep='\t', skiprows=[0], low_memory=False)\
                    .astype({"ftp_path": str})\
                    .assign(database=database, domain=domain)
    asm_summary["fna_path_ncbi"] = asm_summary.apply(
        lambda x: os.path.join(x["ftp_path"], os.path.basename(x["ftp_path"]) + "_genomic.fna.gz"), axis=1)
    
    asm_summary["fna_path_bgi"] = asm_summary.apply(
        lambda x: x["fna_path_ncbi"].replace("ftp://ftp.ncbi.nlm.nih.gov", "/hwfssz1/pub/database/ftp.ncbi.nih.gov"), axis=1)
    
    if check:
        asm_summary["fna_path_bgi_exists"] = asm_summary.apply(
            lambda x: 1 if os.path.exists(x["fna_path_bgi"]) else 0, axis=1)
    return asm_summary


'''
mkdir -p ASSEMBLY_REPORTS

#wget -c -P ASSEMBLY_REPORTS https://ftp.ncbi.nlm.nih.gov/genomes/ASSEMBLY_REPORTS/ANI_report_bacteria.txt
wget -c -P ASSEMBLY_REPORTS https://ftp.ncbi.nlm.nih.gov/genomes/ASSEMBLY_REPORTS/ANI_report_prokaryotes.txt
wget -c -P ASSEMBLY_REPORTS https://ftp.ncbi.nlm.nih.gov/genomes/ASSEMBLY_REPORTS/README_ANI_report_prokaryotes.txt
wget -c -P ASSEMBLY_REPORTS https://ftp.ncbi.nlm.nih.gov/genomes/ASSEMBLY_REPORTS/README_assembly_summary.txt

wget -c -P ASSEMBLY_REPORTS https://ftp.ncbi.nlm.nih.gov/genomes/ASSEMBLY_REPORTS/README_change_notice.txt
wget -c -P ASSEMBLY_REPORTS https://ftp.ncbi.nlm.nih.gov/genomes/ASSEMBLY_REPORTS/prokaryote_type_strain_report.txt
wget -c -P ASSEMBLY_REPORTS https://ftp.ncbi.nlm.nih.gov/genomes/ASSEMBLY_REPORTS/species_genome_size.txt.gz

wget -c -P ASSEMBLY_REPORTS https://ftp.ncbi.nlm.nih.gov/genomes/ASSEMBLY_REPORTS/assembly_summary_genbank.txt
wget -c -P ASSEMBLY_REPORTS https://ftp.ncbi.nlm.nih.gov/genomes/ASSEMBLY_REPORTS/assembly_summary_genbank_historical.txt
wget -c -P ASSEMBLY_REPORTS https://ftp.ncbi.nlm.nih.gov/genomes/ASSEMBLY_REPORTS/assembly_summary_refseq.txt
wget -c -P ASSEMBLY_REPORTS https://ftp.ncbi.nlm.nih.gov/genomes/ASSEMBLY_REPORTS/assembly_summary_refseq_historical.txt

mkdir -p refseq/{archaea,bacteria,fungi,viral}

wget -c -P refseq https://ftp.ncbi.nlm.nih.gov/genomes/refseq/assembly_summary_refseq.txt
wget -c -P refseq https://ftp.ncbi.nlm.nih.gov/genomes/refseq/assembly_summary_refseq_historical.txt

wget -c -P refseq/archaea https://ftp.ncbi.nlm.nih.gov/genomes/refseq/archaea/assembly_summary.txt
wget -c -P refseq/archaea https://ftp.ncbi.nlm.nih.gov/genomes/refseq/archaea/assembly_summary_historical.txt

wget -c -P refseq/bacteria https://ftp.ncbi.nlm.nih.gov/genomes/refseq/bacteria/assembly_summary.txt
wget -c -P refseq/bacteria https://ftp.ncbi.nlm.nih.gov/genomes/refseq/bacteria/assembly_summary_historical.txt

wget -c -P refseq/fungi https://ftp.ncbi.nlm.nih.gov/genomes/refseq/fungi/assembly_summary.txt
wget -c -P refseq/fungi https://ftp.ncbi.nlm.nih.gov/genomes/refseq/fungi/assembly_summary_historical.txt

wget -c -P refseq/viral https://ftp.ncbi.nlm.nih.gov/genomes/refseq/viral/assembly_summary.txt
wget -c -P refseq/viral https://ftp.ncbi.nlm.nih.gov/genomes/refseq/viral/assembly_summary_historical.txt

mkdir -p genbank/{archaea,bacteria,fungi,viral}

wget -c -P genbank https://ftp.ncbi.nlm.nih.gov/genomes/genbank/assembly_summary_genbank.txt
wget -c -P genbank https://ftp.ncbi.nlm.nih.gov/genomes/genbank/assembly_summary_genbank_historical.txt

wget -c -P genbank/archaea https://ftp.ncbi.nlm.nih.gov/genomes/genbank/archaea/assembly_summary.txt
wget -c -P genbank/archaea https://ftp.ncbi.nlm.nih.gov/genomes/genbank/archaea/assembly_summary_historical.txt

wget -c -P genbank/bacteria https://ftp.ncbi.nlm.nih.gov/genomes/genbank/bacteria/assembly_summary.txt
wget -c -P genbank/bacteria https://ftp.ncbi.nlm.nih.gov/genomes/genbank/bacteria/assembly_summary_historical.txt

wget -c -P genbank/fungi https://ftp.ncbi.nlm.nih.gov/genomes/genbank/fungi/assembly_summary.txt
wget -c -P genbank/fungi https://ftp.ncbi.nlm.nih.gov/genomes/genbank/fungi/assembly_summary_historical.txt

wget -c -P genbank/viral https://ftp.ncbi.nlm.nih.gov/genomes/genbank/viral/assembly_summary.txt
wget -c -P genbank/viral https://ftp.ncbi.nlm.nih.gov/genomes/genbank/viral/assembly_summary_historical.txt
'''


asm_summary_genbank_archaea = check_ncbi_assembly_report_on_bgi("genbank/archaea/assembly_summary.txt", "genbank", "archaea")


asm_summary_genbank_bacteria = check_ncbi_assembly_report_on_bgi("genbank/bacteria/assembly_summary.txt", "genbank", "bacteria")


asm_summary_genbank_fungi = check_ncbi_assembly_report_on_bgi("genbank/fungi/assembly_summary.txt", "genbank", "fungi")


asm_summary_genbank_viral = check_ncbi_assembly_report_on_bgi("genbank/viral/assembly_summary.txt", "genbank", "viral")


asm_summary_refseq_archaea = check_ncbi_assembly_report_on_bgi("refseq/archaea/assembly_summary.txt", "refseq", "archaea")


asm_summary_refseq_bacteria = check_ncbi_assembly_report_on_bgi("refseq/bacteria/assembly_summary.txt", "refseq", "bacteria")


asm_summary_refseq_fungi = check_ncbi_assembly_report_on_bgi("refseq/fungi/assembly_summary.txt", "refseq", "fungi")


asm_summary_refseq_viral = check_ncbi_assembly_report_on_bgi("refseq/viral/assembly_summary.txt", "refseq", "viral")


#!/usr/bin/env python3


import os
import sys
import warnings
import subprocess


jobid = sys.argv[1]

out= subprocess.run(['bjobs','-noheader',jobid],stdout=subprocess.PIPE).stdout.decode('utf-8')

state = out.strip().split()[2]


map_state={"PEND":'running',
           "RUN":'running',
           "PROV":"running",
           "WAIT":'running',
           "DONE":'success',
           "":'success'}

print(map_state.get(state,'failed'))



#!/usr/bin/env python3

import os
import argparse
import concurrent.futures
import subprocess
import pandas as pd
import numpy as np
from natsort import index_natsorted


def checkm_prepare(gene_table, batch_num, bins_dir):
    os.makedirs(bins_dir, exist_ok=True)

    table_df = pd.read_csv(gene_table, sep="\t")
    table_df = table_df.sort_values(by="bin_id",
                                    key=lambda x: np.argsort(index_natsorted(table_df["bin_id"])))\
                       .reset_index(drop=True)

    batchid = -1
    if len(table_df) > 0:
        for batch in range(0, len(table_df), batch_num):
            batchid += 1
            table_split = table_df.iloc[batch:batch+batch_num, ]
            table_split.to_csv(os.path.join(bins_dir, f"bins_input.{batchid}.tsv"),
                               sep="\t", index=False, header=None)
    else:
        subprocess.run(f'''touch {os.path.join(bins_dir, "bins_input.0.tsv")}''', shell=True)


def MIMAG_quality_level(row):
    """
    https://doi.org/10.1038/nbt.3893
    """
    if (row["completeness"] > 90.0) and (row["contamination"] < 5.0):
        return "high_quality"
    elif (row["completeness"] > 50.0) and (row["contamination"] < 10.0):
        return "medium_quality"
    else:
        return "low_quality"


def SGB_quality_level(row):
    """
    https://doi.org/10.1016/j.cell.2019.01.001
    """
    if (
        (row["strain_heterogeneity"] < 0.5)
        and (row["completeness"] > 90.0)
        and (row["contamination"] < 5.0)
    ):
        return "high_quality"
    elif (row["completeness"] > 50.0) and (row["contamination"] < 5.0):
        return "medium_quality"
    else:
        return "low_quality"


def quality_score(row):
    """
    https://doi.org/10.1038/s41586-019-0965-1
    """
    return row["completeness"] - 5 * row["contamination"]


def parse_checkm_table(checkm_table):
    if os.path.getsize(checkm_table) > 0:
        checkm_df = pd.read_csv(checkm_table, sep="\t")
        return checkm_df
    else:
        return None


def checkm_reporter(checkm_list, output, threads):
    df_list = []
    with concurrent.futures.ProcessPoolExecutor(max_workers=threads) as executor:
        for df in executor.map(parse_checkm_table, checkm_list):
            if df is not None:
                df_list.append(df)

    df_ = pd.DataFrame(
        columns=[
            "bin_id",
            "marker_lineage",
            "genomes",
            "markers",
            "marker_sets",
            "completeness",
            "contamination",
            "strain_heterogeneity",
            "MIMAG_quality_level",
            "SGB_quality_level",
            "quality_score"])

    if len(df_list) >= 1:
        df_ = pd.concat(df_list).rename(
            columns={
                "Bin Id": "bin_id",
                "Marker lineage": "marker_lineage",
                "# genomes": "genomes",
                "# markers": "markers",
                "# marker sets": "marker_sets",
                "Completeness": "completeness",
                "Contamination": "contamination",
                "Strain heterogeneity": "strain_heterogeneity",
            }
        )

    if not df_.empty:
        df_["MIMAG_quality_level"] = df_.apply(
            lambda x: MIMAG_quality_level(x), axis=1)
        df_["SGB_quality_level"] = df_.apply(
            lambda x: SGB_quality_level(x), axis=1)
        df_["quality_score"] = df_.apply(lambda x: quality_score(x), axis=1)

    if output is not None:
        df_.to_csv(output, sep="\t", index=False)

    return df_


def main():
    parser = argparse.ArgumentParser("CheckM reporter")
    parser.add_argument("--checkm_list", type=str, help="checkm out list")
    parser.add_argument("--output", type=str, required=True,
                        help="checkm output file")
    parser.add_argument(
        "--threads", type=int, default=8, help="threads used on combine CheckM output"
    )
    args = parser.parse_args()

    checkm_list = [l.strip() for l in open(args.checkm_list, "r")]
    checkm_reporter(checkm_list, args.output, args.threads)


if __name__ == "__main__":
    main()



import shlex
from collections import OrderedDict
from itertools import chain
from typing import TextIO, Union, List, Any, Dict

import yaml


class Config:
    def __init__(self, data: Union[dict, None] = None):
        self._data = dict()
        if data is not None:
            for key, value in data.items():
                self._data[key] = self.concatenate_params(value)

    def __bool__(self) -> bool:
        return bool(self._data)

    def __contains__(self, item) -> bool:
        return item in self._data

    def get(self, key: str, default: Any = None) -> Any:
        return self._data.get(key, default)

    @staticmethod
    def args_to_dict(args: str) -> Dict[str, str]:
        """Converts a string into a dictionary where key/value pairs are consecutive
        elements of the string.
        Eg '-J "2" -q 3' --> {'-J': '2', '-q': '3'}
        """
        args_iter = shlex.shlex(args, posix=True)
        args_iter.whitespace_split = True
        return OrderedDict(zip(args_iter, args_iter))

    @staticmethod
    def concatenate_params(params: Union[List[str], str]) -> str:
        if isinstance(params, str):
            return params
        return " ".join(filter(None, params))

    def default_params(self) -> str:
        return self.get("__default__", "")

    def params_for_rule(self, rulename: str) -> str:
        """Loads default + rule-specific arguments.
        Arguments specified for a rule override default-specified arguments.
        Shlex-joining is required to properly pass quoted escapes in yaml
        to the shell.
        """
        default_params = self.args_to_dict(self.default_params())
        rule_params = self.args_to_dict(self.get(rulename, ""))
        default_params.update(rule_params)
        return " ".join(map(shlex.quote, chain.from_iterable(default_params.items())))

    @staticmethod
    def from_stream(stream: TextIO) -> "Config":
        data = yaml.safe_load(stream)
        return Config(data)



#!/usr/bin/env python3
import math
import re
import subprocess
import sys
from pathlib import Path
from typing import List, Union, Optional

from snakemake.utils import read_job_properties

if not __name__.startswith("tests.src."):
    sys.path.append(str(Path(__file__).parent.absolute()))
    from OSLayer import OSLayer
    from CookieCutter import CookieCutter
    from lsf_config import Config
    from memory_units import Unit, Memory
else:
    from .OSLayer import OSLayer
    from .CookieCutter import CookieCutter
    from .lsf_config import Config
    from .memory_units import Unit, Memory

PathLike = Union[str, Path]


class BsubInvocationError(Exception):
    pass


class JobidNotFoundError(Exception):
    pass


class Submitter:
    def __init__(
        self,
        jobscript: PathLike,
        cluster_cmds: List[str] = None,
        memory_units: Unit = Unit.MEGA,
        lsf_config: Optional[Config] = None,
    ):
        if cluster_cmds is None:
            cluster_cmds = []
        if lsf_config is None:
            lsf_config = Config()

        self._jobscript = jobscript
        self._cluster_cmd = " ".join(cluster_cmds)
        self._job_properties = read_job_properties(self._jobscript)
        self.random_string = OSLayer.get_uuid4_string()
        self._memory_units = memory_units
        self.lsf_config = lsf_config

    @property
    def jobscript(self) -> str:
        return self._jobscript

    @property
    def job_properties(self) -> dict:
        return self._job_properties

    @property
    def cluster(self) -> dict:
        return self.job_properties.get("cluster", dict())

    @property
    def threads(self) -> int:
        return self.job_properties.get("threads", CookieCutter.get_default_threads())

    @property
    def resources(self) -> dict:
        return self.job_properties.get("resources", dict())

    @property
    def mem_mb(self) -> Memory:
        mem_value = self.resources.get(
            "mem_mb", self.cluster.get("mem_mb", CookieCutter.get_default_mem_mb())
        )
        return Memory(mem_value, unit=Unit.MEGA)

    @property
    def memory_units(self) -> Unit:
        return self._memory_units

    @property
    def resources_cmd(self) -> str:
        mem_in_clusters_units = self.mem_mb.to(self.memory_units)
        mem_value_to_submit = math.ceil(mem_in_clusters_units.value)
        resources_str = (
            "-M {mem} -n {threads} -R 'select[mem>{mem}] "
            "rusage[mem={mem}] span[hosts=1]'".format(
                mem=mem_value_to_submit, threads=self.threads
            )
        )

        for time_str in ("time", "runtime", "walltime", "time_min"):
            if self.resources.get(time_str, False):
                resources_str += " -W {}".format(self.resources[time_str])
        return resources_str

    @property
    def wildcards(self) -> dict:
        return self.job_properties.get("wildcards", dict())

    @property
    def wildcards_str(self) -> str:
        return (
            ".".join("{}={}".format(k, v) for k, v in self.wildcards.items())
            or "unique"
        )

    @property
    def rule_name(self) -> str:
        if not self.is_group_jobtype:
            return self.job_properties.get("rule", "rule_name")
        return self.groupid

    @property
    def groupid(self) -> str:
        return self.job_properties.get("groupid", "group")

    @property
    def is_group_jobtype(self) -> bool:
        return self.job_properties.get("type", "") == "group"

    @property
    def jobname(self) -> str:
        if self.is_group_jobtype:
            return "{groupid}_{jobid}".format(groupid=self.groupid, jobid=self.jobid)
        return self.cluster.get(
            "jobname",
            "{rule_name}.{wildcards_str}".format(
                rule_name=self.rule_name, wildcards_str=self.wildcards_str
            ),
        )

    @property
    def jobid(self) -> str:
        if self.is_group_jobtype:
            return self.job_properties.get("jobid", "").split("-")[0]
        return str(self.job_properties.get("jobid"))

    @property
    def logdir(self) -> Path:
        project_logdir = Path(self.cluster.get("logdir", CookieCutter.get_log_dir()))
        return project_logdir / self.rule_name / self.wildcards_str

    @property
    def outlog(self) -> Path:
        return self.logdir / "jobid{jobid}_{random_string}.out".format(
            jobid=self.jobid, random_string=self.random_string
        )

    @property
    def errlog(self) -> Path:
        return self.logdir / "jobid{jobid}_{random_string}.err".format(
            jobid=self.jobid, random_string=self.random_string
        )

    @property
    def jobinfo_cmd(self) -> str:
        return '-o "{out_log}" -e "{err_log}" -J "{jobname}"'.format(
            out_log=self.outlog, err_log=self.errlog, jobname=self.jobname
        )

    @property
    def queue(self) -> str:
        return self.cluster.get("queue", CookieCutter.get_default_queue())

    @property
    def queue_cmd(self) -> str:
        return "-q {}".format(self.queue) if self.queue else ""

    @property
    def rule_specific_params(self) -> str:
        return self.lsf_config.params_for_rule(self.rule_name)

    @property
    def cluster_cmd(self) -> str:
        return self._cluster_cmd

    @property
    def submit_cmd(self) -> str:
        params = [
            "bsub",
            self.resources_cmd,
            self.jobinfo_cmd,
            self.queue_cmd,
            self.cluster_cmd,
            self.rule_specific_params,
            self.jobscript,
        ]
        return " ".join(p for p in params if p)

    def _create_logdir(self):
        OSLayer.mkdir(self.logdir)

    def _remove_previous_logs(self):
        OSLayer.remove_file(self.outlog)
        OSLayer.remove_file(self.errlog)

    def _submit_cmd_and_get_external_job_id(self) -> int:
        output_stream, error_stream = OSLayer.run_process(self.submit_cmd)
        match = re.search(r"Job <(\d+)> is submitted", output_stream)
        jobid = match.group(1)
        return int(jobid)

    def _get_parameters_to_status_script(self, external_job_id: int) -> str:
        return "{external_job_id} {outlog}".format(
            external_job_id=external_job_id, outlog=self.outlog
        )

    def submit(self):
        self._create_logdir()
        self._remove_previous_logs()
        try:
            external_job_id = self._submit_cmd_and_get_external_job_id()
            parameters_to_status_script = self._get_parameters_to_status_script(
                external_job_id
            )
            OSLayer.print(parameters_to_status_script)
        except subprocess.CalledProcessError as error:
            raise BsubInvocationError(error)
        except AttributeError as error:
            raise JobidNotFoundError(error)


if __name__ == "__main__":
    workdir = Path().resolve()
    config_file = workdir / "lsf.yaml"
    if config_file.exists():
        with config_file.open() as stream:
            lsf_config = Config.from_stream(stream)
    else:
        lsf_config = Config()

    jobscript = sys.argv[-1]
    cluster_cmds = sys.argv[1:-1]
    memory_units = Unit.from_suffix(CookieCutter.get_lsf_unit_for_limits())
    lsf_submit = Submitter(
        jobscript=jobscript,
        memory_units=memory_units,
        lsf_config=lsf_config,
        cluster_cmds=cluster_cmds,
    )
    lsf_submit.submit()



#!/usr/bin/env python3

import os
import pandas as pd
import concurrent.futures


def gen_samples_info(samples, output, config):
    """
    GSC MIxS environmental sample template
    * fileds are mandatory
    """

    header = [
        "*sample_name",
        "sample_title",
        "*tax_id",
        "*organism",
        "*investigation_type",
        "*collection_date",
        "*env_biome",
        "*env_feature",
        "*env_material",
        "*geo_loc_name",
        "*host",
        "*lat_lon",
        "*strain",
        "*estimated_size",
        "*isol_growth_condt",
        "*num_replicons",
        "*ref_biomaterial",
        "*ploidy",
        "*propagation",
        "biotic_relationship",
        "hem_administration" "encoded_traits",
        "ethnicity",
        "extrachrom_elements",
        "health_state",
        "host_age",
        "host_body_mass_index",
        "host_body_product",
        "host_body_temp",
        "host_diet",
        "host_disease",
        "host_family_relationship",
        "host_genotype",
        "host_height",
        "host_last_meal",
        "host_occupation",
        "host_phenotype",
        "host_pulse",
        "host_sex",
        "host_subject_id",
        "host_taxid",
        "host_tissue_sampled",
        "host_tot_mass",
        "ihmc_medication_code",
        "isolation_source",
        "medic_hist_perform",
        "misc_param",
        "nose_mouth_teeth_throat_disord",
        "organism_count",
        "oxy_stat_samp",
        "pathogenicity",
        "perturbation",
        "rel_to_oxygen",
        "samp_collect_device",
        "samp_mat_process",
        "samp_salinity",
        "samp_size",
        "samp_store_dur",
        "samp_store_loc",
        "samp_store_temp",
        "samp_vol_we_dna_ext",
        "source_material_id",
        "subspecf_gen_lin",
        "temp",
        "time_last_toothbrush",
        "trophic_level",
        "description",
    ]

    samples_df = pd.DataFrame(columns=header)
    samples_df["*sample_name"] = samples.index.get_level_values("sample_id").unique()
    samples_df["sample_title"] = samples.index.get_level_values("sample_id").unique()
    samples_df["source_material_id"] = samples.index.get_level_values("sample_id").unique()
    for key in config["upload"]["samples"]:
        samples_df["*" + key] = config["upload"]["samples"][key]
    samples_df.to_excel(output, index=False, startrow=12)


def parse_md5(md5_file):
    try:
        if os.path.exists(md5_file):
            df = pd.read_csv(
                md5_file, sep="\s+", header=None, names=["file_md5", "file_name"]
            )
            if df.empty:
                print("%s is empty, please check" % md5_file)
                return None
            df["sample_name"] = df.apply(
                lambda x: os.path.basename(x["file_name"]).split(".")[0], axis=1
            )
            df["file_name"] = df.apply(
                lambda x: os.path.basename(x["file_name"]), axis=1
            )
            if len(df) == 2:
                df_fq1 = df.iloc[0].to_frame().T
                df_fq2 = (
                    df.iloc[1]
                    .to_frame()
                    .T.rename(
                        columns={"file_name": "file2_name", "file_md5": "file2_md5"}
                    )
                )
                return pd.merge(df_fq1, df_fq2)
            else:
                df["file2_name"] = ""
                df["file2_md5"] = ""
                return df
        else:
            print("%s is not exists" % md5_file)
            return None
    except pd.errors.EmptyDataError:
        print("%s is empty, please check" % md5_file)
        return None


def gen_info(input_list, output, config, workers, group):
    df_list = []
    with concurrent.futures.ProcessPoolExecutor(max_workers=workers) as executor:
        for df_ in executor.map(parse_md5, input_list):
            if df_ is not None:
                df_list.append(df_)
    df = pd.concat(df_list)

    for key in config["upload"][group].keys():
        df[key] = config["upload"][group][key]

    df["project_accession"] = config["upload"]["project_accession"]

    if group == "sequencing_run":
        run_df = df.loc[
            :,
            ["project_accession", "sample_name"]
            + list(config["upload"][group].keys())
            + ["file_name", "file_md5", "file2_name", "file2_md5"],
        ]
        if config["params"]["reads_layout"] == "pe":
            run_df["library_layout"] = "paired"
        else:
            run_df["library_layout"] = "fragment/single"
            run_df["nominal_size"] = ""

        run_df["library_name"] = run_df["sample_name"]

        run_df = run_df.rename(
            columns={
                "project_accession": "*project_accession",
                "sample_name": "*sample_name",
                "experiment_title": "*experiment_title",
                "library_name": "*library_name",
                "library_strategy": "*library_strategy",
                "library_selection": "*library_selection",
                "library_layout": "*library_layout",
                "platform": "*platform",
                "instrument_model": "*instrument_model",
                "spot_layout": "*spot_layout",
                "file_type": "*file_type",
                "file_name": "*file_name",
                "file_md5": "*file_md5",
                "file2_name": "*file2_name",
                "file2_md5": "*file2_md5",
            }
        )
        run_df.to_excel(output, sheet_name="Metadata", index=False)

    if group == "assembly":
        asm_df = df.rename(
            columns={"file_name": "fasta_file_name", "file_md5": "fasta_file_md5"}
        )
        asm_df["assembly_name"] = df["sample_name"] + "-" + df["assembly_method"]
        asm_df["sample_accession"] = ""
        asm_df = asm_df.loc[
            :,
            ["project_accession", "sample_accession", "sample_name", "assembly_name"]
            + list(config["upload"][group].keys())
            + ["fasta_file_name", "fasta_file_md5"],
        ]
        asm_df.to_excel(output, sheet_name="Genome_Assembly", index=False)



import glob
import os
import time
import subprocess

bin_list = glob.glob(snakemake.input["bins_dir"] + "/*.fa")
gff_count = 0

for bin_fa in bin_list:
    bin_id = os.path.basename(os.path.splitext(bin_fa)[0])
    output_dir = os.path.join(snakemake.params["output_dir"], bin_id)
    gff_file = os.path.join(output_dir, bin_id + ".gff")

    subprocess(f'''echo "\nProcessing {bin_fa}\n" >> {snakemake.log}''', shell=True)
    subprocess(
        f'''
        prokka {bin_fa} \
        --force \
        --centre X \
        --compliant \
        --cpus {snakemake.threads} \
        --outdir {output_dir} \
        --locustag {bin_id} \
        --prefix {bin_id} \
        --kingdom {snakemake.params["kingdom"]} \
        2>> {snakemake.log} 
        ''', shell=True)

    if os.path.exists(gff_file):
        gff_count += 1

if gff_count == len(bin_list):
    subprocess('''touch {snakemake.output["done"]}''', shell=True)




import re
from enum import Enum
from typing import Union
from collections import namedtuple


class InvalidSuffix(Exception):
    pass


class InvalidPower(Exception):
    pass


class InvalidMemoryString(Exception):
    pass


Scale = namedtuple("Scale", ["power", "metric_suffix"])


SCALE_MAP = {
    "B": Scale(0, "B"),
    "K": Scale(1, "KB"),
    "M": Scale(2, "MB"),
    "G": Scale(3, "GB"),
    "T": Scale(4, "TB"),
    "P": Scale(5, "PB"),
    "E": Scale(6, "EB"),
    "Z": Scale(7, "ZB"),
}


class Unit(Enum):
    BYTES = SCALE_MAP["B"]
    KILO = SCALE_MAP["K"]
    MEGA = SCALE_MAP["M"]
    GIGA = SCALE_MAP["G"]
    TERA = SCALE_MAP["T"]
    PETA = SCALE_MAP["P"]
    EXA = SCALE_MAP["E"]
    ZETTA = SCALE_MAP["Z"]

    @staticmethod
    def from_suffix(suffix: str) -> "Unit":
        first_letter = suffix[0].upper()
        if first_letter not in SCALE_MAP:
            valid_suffixes = ",".join(
                scale.metric_suffix for scale in SCALE_MAP.values()
            )
            raise InvalidSuffix(
                "{suffix}. Valid suffixes are: {valid_suffixes}".format(
                    suffix=suffix, valid_suffixes=valid_suffixes
                )
            )
        return Unit(SCALE_MAP[first_letter])

    @staticmethod
    def from_power(power: int) -> "Unit":
        valid_powers = []
        for scale in SCALE_MAP.values():
            if scale.power == power:
                return Unit(scale)
            valid_powers.append(scale.power)

        raise InvalidPower(
            "{power}. Valid powers are: {valid}".format(
                power=power, valid=",".join(str(p) for p in valid_powers)
            )
        )

    @property
    def power(self) -> int:
        return self.value.power

    @property
    def suffix(self) -> str:
        return self.value.metric_suffix


Number = Union[int, float]


class Memory:
    def __init__(self, value: Number = 1, unit: Unit = Unit.BYTES):
        self.value = value
        self.unit = unit
        self._decimal_scaling_factor = 1000
        self._binary_scaling_factor = 1024

    def __eq__(self, other: "Memory") -> bool:
        return self.bytes() == other.bytes()

    def __repr__(self) -> str:
        val = (
            int(self.value)
            if isinstance(self.value, int) or self.value.is_integer()
            else self.value
        )
        return "{val}{sfx}".format(val=val, sfx=self.suffix)

    @property
    def power(self) -> int:
        return self.unit.power

    @property
    def suffix(self) -> str:
        return self.unit.suffix

    def _scaling_factor(self, decimal: bool = True) -> int:
        return self._decimal_scaling_factor if decimal else self._binary_scaling_factor

    def bytes(self, decimal_multiples: bool = True) -> float:
        scaling_factor = self._scaling_factor(decimal_multiples)
        return float(self.value * (scaling_factor ** self.power))

    def to(self, unit: Unit, decimal_multiples: bool = True) -> "Memory":
        scaling_factor = self._scaling_factor(decimal_multiples) ** unit.power
        size = self.bytes(decimal_multiples)
        size /= scaling_factor

        return Memory(size, unit)

    @staticmethod
    def from_str(s: str) -> "Memory":
        valid_suffixes = "".join(scale.metric_suffix for scale in SCALE_MAP.values())
        regex = re.compile(
            r"^(?P<size>[0-9]*\.?[0-9]+)\s*(?P<sfx>[{}]B?)?$".format(valid_suffixes),
            re.IGNORECASE,
        )
        match = regex.search(s)

        if not match:
            raise InvalidMemoryString("{s} is an invalid memory string.".format(s=s))

        size = float(match.group("size"))
        suffix = match.group("sfx") or "B"
        unit = Unit.from_suffix(suffix)

        return Memory(size, unit)



#!/usr/bin/env python

import os
import re
import sys
import argparse
from Bio import SeqIO


def parse_gff(gff_file, min_len):
    '''
    extract pep id under the requirement of the minimal contig length cutoff from the GFF file generated by prodigal 
    '''
    save = False
    min_len = int(min_len)
    pep_id_list = []
    with open(gff_file, "r") as ih:
        for line in ih:
            seq_len = 0

            if line.startswith("##") or line.startswith("# Model Data") or line.strip() == '"':
                continue

            elif line.startswith("# Sequence Data"):
                line_split = line.strip().split(";")
                for token in line_split:
                    if "seqlen=" in token:
                        seq_len = int(token[token.find("=") + 1:])
                if seq_len < min_len:
                    save = False 
                else:
                    save = True
            elif save:
                line_split = re.split("\s+", line.strip())
                seq_id = line_split[0]
                trans_id = line_split[-1].split(";")[0].split("_")[-1]
                pep_id = f'''{seq_id}_{trans_id}'''
                pep_id_list.append(pep_id)
            else:
                continue
    return pep_id_list


def extract_faa(faa_file, pep_id_list, out_file, assembly_group=None):
    if os.path.dirname(out_file) != "":
        os.makedirs(os.path.dirname(out_file), exist_ok=True)

    with open(out_file, "w") as oh:
        for seq in SeqIO.parse(faa_file, "fasta"):
            if seq.id in pep_id_list:
                if assembly_group is not None:
                    seq.id = f'''{assembly_group}C{seq.id}'''
                    seq.name = f'''{assembly_group}C{seq.name}'''
                    seq.description = f'''{assembly_group}C{seq.description}'''
                SeqIO.write(seq, oh, "fasta")


def main():
    parser = argparse.ArgumentParser("PEP extractor")
    parser.add_argument("--faa-file", dest="faa_file", type=str, required=True, help="protein file")
    parser.add_argument("--gff-file", dest="gff_file", type=str, required=True, help="gff file")
    parser.add_argument("--min-contig", dest="min_contig", default=2000, type=int, help="minimal contig length, default: 2000")
    parser.add_argument("--out-file", dest="out_file", type=str, required=True, help="output protein file")
    args = parser.parse_args()

    pep_id_list = parse_gff(args.gff_file, args.min_contig)
    if len(pep_id_list) > 0:
        extract_faa(args.faa_file, pep_id_list, args.out_file)
    else:
        sys.exit("Emplty protein file after contigs length control")


if __name__ == '__main__':
    main()


#!/usr/bin/env python
import os
import sys
import subprocess


with os.scandir(sys.argv[1]) as itr:
    for entry in itr:
        bin_id, bin_suffix = os.path.splitext(entry.name)
        bin_name, cluster_num = bin_id.rsplit(".", maxsplit=1)
        bin_id = bin_name + "." + cluster_num.lstrip("0")
        if bin_suffix == ".fasta":
            subprocess.run('''mv %s %s''' \
                  % (os.path.join(sys.argv[1], entry.name),
                     os.path.join(sys.argv[1], bin_id + ".fa")), shell=True)





#!/usr/bin/env python3

import os
import gzip
import sys
import subprocess
import pandas as pd
from Bio import SeqIO


def parse_genomes(samples_tsv, output_dir, check_samples=False):
    header = ["id", "genome", "abundance", "reads_num", "model"]

    genomes_df = pd.read_csv(samples_tsv, sep="\t").set_index("id", drop=False)

    cancel = False
    for i in header:
        if i not in genomes_df.columns:
            cancel = True
            print(f'Error: {i} not in {genomes_df.columns} header')

    for i in genomes_df.index.unique():
        if "." in i:
            cancel = True
            print('Error: sample id %s contains ".", please remove all "."' % i)

    if cancel:
        sys.exit(1)

    genomes_df["fq1"] = genomes_df.apply(
        lambda x: os.path.join(
            output_dir, "short_reads/%s.simulate.1.fq.gz" % x["id"],
        ),
        axis=1,
    )
    genomes_df["fq2"] = genomes_df.apply(
        lambda x: os.path.join(
            output_dir, "short_reads/%s.simulate.2.fq.gz" % x["id"],
        ),
        axis=1,
    )
    return genomes_df


def simulate_short_reads(
    genomes, output_prefix, r1, r2, abunf, model, reads_num, abundance, threads, logf,
):
    if len(abundance) != 0:
        with open(abunf, "w") as outh:
            for (g, a) in zip(genomes, abundance):
                inh = gzip.open(g, "rt") if g.endswith(".gz") else open(g, "r")
                genome = []
                total_len = 0
                for record in SeqIO.parse(inh, "fasta"):
                    total_len += len(record.seq)
                    genome.append((record.id, len(record.seq)))
                for s in genome:
                    outh.write("%s\t%f\n" %
                               (s[0], float(a) * s[1] / total_len))
                inh.close()

    args = (
        ["iss", "generate", "--cpus", str(threads), "--genomes"]
        + genomes
        + ["--n_reads", reads_num, "--model", model, "--output", output_prefix]
    )

    if len(abundance) != 0:
        args += ["--abundance_file", abunf]
    print(" ".join(args))
    env = os.environ.copy()
    proc = subprocess.Popen(
        args, stdout=subprocess.PIPE, stderr=subprocess.PIPE, env=env, encoding="utf-8",
    )
    output, error = proc.communicate()

    with open(logf, "w") as logh:
        logh.write(error)

    if proc.returncode == 0:
        if len(abundance) == 0:
            default_abunf = output_prefix + "_abundance.txt"
            if os.path.exists(default_abunf):
                os.rename(default_abunf, abunf)
        subprocess.run(f"pigz -p {threads} {output_prefix}_R1.fastq", shell=True)
        subprocess.run(f"pigz -p {threads} {output_prefix}_R2.fastq", shell=True)
        os.rename(f"{output_prefix}_R1.fastq.gz", r1)
        os.rename(f"{output_prefix}_R2.fastq.gz", r2)
    else:
        sys.exit(1)


def get_simulate_info(genomes_df, wildcards, col):
    return genomes_df.loc[[wildcards.sample], col].dropna().tolist()



#
# Based on lsf CookieCutter.py
#
import os
import json

d = os.path.dirname(__file__)
with open(os.path.join(d, "settings.json")) as fh:
    settings = json.load(fh)


class CookieCutter:

    SBATCH_DEFAULTS = settings['SBATCH_DEFAULTS']
    CLUSTER_NAME = settings['CLUSTER_NAME']
    CLUSTER_CONFIG = settings['CLUSTER_CONFIG']
    ADVANCED_ARGUMENT_CONVERSION = settings['ADVANCED_ARGUMENT_CONVERSION']

    @staticmethod
    def get_cluster_option() -> str:
        cluster = CookieCutter.CLUSTER_NAME
        if cluster != "":
            return f"--cluster={cluster}"
        return ""

    @staticmethod
    def get_advanced_argument_conversion() -> bool:
        val = {"yes": True, "no": False}[
            CookieCutter.ADVANCED_ARGUMENT_CONVERSION
        ]
        return val



#!/usr/bin/env python
import os
import sys
from setuptools import setup

exec(open("metapi/__about__.py").read())

if sys.argv[-1] == "publish":
    os.system("python setup.py sdist upload")
    sys.exit()

with open("README.md") as f:
    long_description = f.read()

packages = ["metapi"]
package_data = {
    "metapi": [
        "metapi/config/*.yaml",
        "metapi/envs/*.yaml",
        "metapi/snakefiles/*.smk",
        "metapi/rules/*.smk",
        "metapi/wrappers/*.py",
        "metapi/*.py",
    ]
}
data_files = [(".", ["LICENSE", "README.md"])]

entry_points = {"console_scripts": ["metapi=metapi.corer:main"]}

requires = [
    req.strip()
    for req in open("requirements.txt", "r").readlines()
    if not req.startswith("#")
]

classifiers = [
    "Development Status :: 3 - Alpha",
    "Environment :: Console",
    "Intended Audience :: Developers",
    "Intended Audience :: Science/Research",
    "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
    "Natural Language :: English",
    "Operating System :: OS Independent",
    "Programming Language :: Python :: 3.7",
    "Programming Language :: Python :: 3.8",
    "Programming Language :: Python :: 3.9",
    "Programming Language :: Python :: 3.10",
    "Topic :: Scientific/Engineering :: Bio-Informatics",
]

setup(
    name="metapi",
    version=__version__,
    author=__author__,
    author_email="alienchuj@gmail.com",
    url="https://github.com/ohmeta/metapi",
    description="a pipeline to construct a genome catalogue from metagenomics data",
    long_description_content_type="text/markdown",
    long_description=long_description,
    entry_points=entry_points,
    packages=packages,
    package_data=package_data,
    data_files=data_files,
    include_package_data=True,
    install_requires=requires,
    license="GPLv3+",
    classifiers=classifiers,
)



#!/usr/bin/env python

import argparse
import os
import sys
import shutil

from ruamel.yaml import YAML


def parse_yaml(yaml_file):
    yaml = YAML()
    with open(yaml_file, "r") as f:
        return yaml.load(f)


def update_config(yaml_file_old, yaml_file_new, yaml_content, remove=True):
    yaml = YAML()
    yaml.default_flow_style = False
    if remove:
        os.remove(yaml_file_old)
    with open(yaml_file_new, "w") as f:
        yaml.dump(yaml_content, f)


class metaconfig:
    """
    config project directory
    """

    sub_dirs = [
        "envs",
        "profiles",
        "results",
        "logs/00.simulate_short_reads",
        "logs/00.prepare_short_reads",
        "logs/00.prepare_long_reads",
        "logs/00.raw_fastqc",
        "logs/00.raw_fastqc_multiqc",
        "logs/00.raw_report",
        "logs/00.raw_report_refine",
        "logs/00.raw_report_merge",
        "logs/01.trimming_sickle",
        "logs/01.trimming_fastp",
        "logs/01.trimming_fastp_multiqc",
        "logs/01.trimming_report",
        "logs/01.trimming_report_refine",
        "logs/01.trimming_report_merge",
        "logs/02.rmhost_bwa_index",
        "logs/02.rmhost_bwa",
        "logs/02.rmhost_bowtie2_index",
        "logs/02.rmhost_bowtie2",
        "logs/02.rmhost_minimap2_index",
        "logs/02.rmhost_minimap2",
        "logs/02.rmhost_kraken2",
        "logs/02.rmhost_kneaddata",
        "logs/02.rmhost_alignment_report",
        "logs/02.rmhost_report",
        "logs/02.rmhost_report_refine",
        "logs/02.rmhost_report_merge",
        "logs/04.assembly_megahit",
        "logs/04.assembly_idba_ud",
        "logs/04.assembly_metaspades",
        "logs/04.assembly_spades",
        "logs/04.assembly_plass",
        "logs/04.assembly_opera_ms",
        "logs/04.assembly_metaquast",
        "logs/04.assembly_metaquast_multiqc",
        "logs/04.assembly_report_merge",
        "logs/05.alignment_scaftigs_index",
        "logs/05.alignment_reads_scaftigs",
        "logs/05.alignment_base_depth",
        "logs/06.binning_metabat2_coverage",
        "logs/06.binning_metabat2",
        "logs/06.binning_maxbin2_coverage",
        "logs/06.binning_maxbin2",
        "logs/06.binning_concoct_cut_bed",
        "logs/06.binning_concoct_coverage",
        "logs/06.binning_concoct",
        "logs/06.binning_graphbin2_prepare_assembly",
        "logs/06.binning_graphbin2",
        "logs/06.binning_dastools",
        "logs/06.binning_vamb_combine_scaftigs",
        "logs/06.binning_vamb_dict_scaftigs",
        "logs/06.binning_vamb_index_scaftigs",
        "logs/06.binning_vamb_align_scaftigs",
        "logs/06.binning_vamb_sort_bam",
        "logs/06.binning_vamb_align_scaftigs_report",
        "logs/06.binning_vamb_coverage",
        "logs/06.binning_vamb_gen_abundance_matrix",
        "logs/06.binning_vamb",
        "logs/06.binning_report",
        "logs/06.binning_report_merge",
        "logs/06.identify_virsorter2_prepare",
        "logs/06.identify_virsorter2",
        "logs/06.identify_virsorter2_merge",
        "logs/06.identify_deepvirfinder",
        "logs/06.identify_phamb_micomplete",
        "logs/06.identify_phamb_vog",
        "logs/06.identify_phamb_randomforest",
        "logs/07.predict_scaftigs_gene_prodigal",
        "logs/07.predict_scaftigs_gene_prokka",
        "logs/07.predict_scaftigs_gene_prokka_multiqc",
        "logs/07.predict_bins_gene_prodigal",
        "logs/07.predict_bins_gene_prokka",
        "logs/07.predict_bins_gene_prokka_multiqc",
        "logs/08.checkm_lineage_wf",
        "logs/09.dereplicate_mags_drep",
        "logs/09.dereplicate_gene_prepare",
        "logs/09.dereplicate_gene_cdhit",
        "logs/10.taxonomic_gtdbtk",
        "logs/99.upload_md5_short_reads",
        "logs/99.upload_md5_scaftigs"
    ]

    def __init__(self, work_dir):
        self.work_dir = os.path.realpath(work_dir)
        self.metapi_dir = os.path.dirname(os.path.abspath(__file__))

        self.config_file = os.path.join(self.metapi_dir, "config", "config.yaml")
        self.envs_dir = os.path.join(self.metapi_dir, "envs")
        self.profiles_dir = os.path.join(self.metapi_dir, "profiles")
        self.new_config_file = os.path.join(self.work_dir, "config.yaml")

    def __str__(self):
        message = """
.___  ___.  _______ .___________.    ___      .______    __
|   \/   | |   ____||           |   /   \     |   _  \  |  |
|  \  /  | |  |__   `---|  |----`  /  ^  \    |  |_)  | |  |
|  |\/|  | |   __|      |  |      /  /_\  \   |   ___/  |  |
|  |  |  | |  |____     |  |     /  _____  \  |  |      |  |
|__|  |__| |_______|    |__|    /__/     \__\ | _|      |__|

           Omics for All, Open Source for All

A general metagenomics data mining system focus on robust microbiome research.

Thanks for using metapi.

A metagenomics project has been created at %s


if you want to create fresh conda environments:

        metapi mag_wf --conda-create-envs-only
        metapi gene_wf --conda-create-envs-only

if you have environments:

        metapi mag_wf --help
        metapi gene_wf --help
""" % (
            self.work_dir
        )

        return message

    def create_dirs(self):
        """
        create project directory
        """
        if not os.path.exists(self.work_dir):
            os.mkdir(self.work_dir)

        for sub_dir in metaconfig.sub_dirs:
            os.makedirs(os.path.join(self.work_dir, sub_dir), exist_ok=True)

        for i in os.listdir(self.envs_dir):
            dest_file = os.path.join(self.work_dir, "envs", i)
            if os.path.exists(dest_file):
                print(f"{dest_file} exists, please remove or backup it first")
                sys.exit(-1)
            else:
                shutil.copyfile(os.path.join(self.envs_dir, i), dest_file)

        for i in os.listdir(self.profiles_dir):
            dest_dir = os.path.join(self.work_dir, "profiles", i)
            if os.path.exists(dest_dir):
                print(f"{dest_dir} exists, please remove or backup it first")
                sys.exit(-1)
            else:
                shutil.copytree(os.path.join(self.profiles_dir, i), dest_dir)

    def get_config(self):
        """
        get default configuration
        """
        return parse_yaml(self.config_file)


# https://github.com/Ecogenomics/CheckM/blob/master/checkm/customHelpFormatter.py
class custom_help_formatter(argparse.HelpFormatter):
    """Provide a customized format for help output.
    http://stackoverflow.com/questions/9642692/argparse-help-without-duplicate-allcaps
    """

    def _split_lines(self, text, width):
        return text.splitlines()

    def _get_help_string(self, action):
        h = action.help
        if "%(default)" not in action.help:
            if (
                action.default != ""
                and action.default != []
                and action.default != None
                and action.default != False
            ):
                if action.default is not argparse.SUPPRESS:
                    defaulting_nargs = [
                        argparse.OPTIONAL, argparse.ZERO_OR_MORE]

                    if action.option_strings or action.nargs in defaulting_nargs:
                        if "\n" in h:
                            lines = h.splitlines()
                            lines[0] += " (default: %(default)s)"
                            h = "\n".join(lines)
                        else:
                            h += " (default: %(default)s)"
            return h

    def _fill_text(self, text, width, indent):
        return "".join([indent + line for line in text.splitlines(True)])

    def _format_action_invocation(self, action):
        if not action.option_strings:
            default = self._get_default_metavar_for_positional(action)
            (metavar,) = self._metavar_formatter(action, default)(1)
            return metavar

        else:
            parts = []

            # if the Optional doesn't take a value, format is:
            #    -s, --long
            if action.nargs == 0:
                parts.extend(action.option_strings)

            # if the Optional takes a value, format is:
            #    -s ARGS, --long ARGS
            else:
                default = self._get_default_metavar_for_optional(action)
                args_string = self._format_args(action, default)
                for option_string in action.option_strings:
                    parts.append(option_string)

                return "%s %s" % (", ".join(parts), args_string)

            return ", ".join(parts)

    def _get_default_metavar_for_optional(self, action):
        return action.dest.upper()

    def _get_default_metavar_for_positional(self, action):
        return action.dest
